﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Threading.Tasks;
using EdFi.OdsApi.Sdk.Api.Immunizations;
using EdFi.OdsApi.Sdk.Api.Resources;
using EdFi.OdsApi.Sdk.Models.Resources;
using EdFi.OdsApi.Sdk.Api.Identities;
using EdFi.OdsApi.Sdk.Models.Identities;

namespace EdFi.OdsApi.Sdk.Client
{
    class Program
    {
        static void Main(string[] args)
        {
            var yourApiKey = "";
            var yourApiSecret = "";
            var baseUrl = "https://uaapps.dpi.wi.gov/EdFiWebAPI/";

            System.Net.ServicePointManager.ServerCertificateValidationCallback = (sender, certificate, chain, sslPolicyErrors) => true;
            var tokenRetriever = new TokenRetriever($"{baseUrl}", yourApiKey, yourApiSecret);
            
            var token = tokenRetriever.ObtainNewBearerToken();

            var configuration = new Configuration()
            {
                AccessToken = token,
                BasePath = $"{baseUrl}data/v3/2021"
            };

            var schoolsApi = new SchoolsApi(configuration);
            var schoolsResponse = schoolsApi.GetSchoolsWithHttpInfo(localEducationAgencyId: 7533);

            var allStaff = new ConcurrentBag<EdFiStaff>();
            Parallel.ForEach(schoolsResponse.Data, (school) =>
            {
                var staffSectionsApi = new StaffSectionAssociationsApi(configuration);
                var staffSectionsResponse = staffSectionsApi.GetStaffSectionAssociationsWithHttpInfo(schoolId: school.SchoolId);

                Parallel.ForEach(staffSectionsResponse.Data, (staff) =>
                {
                    var staffsApi = new StaffsApi(configuration);
                    var staffsResponse = staffsApi.GetStaffsWithHttpInfo(staffUniqueId: staff.StaffReference.StaffUniqueId);

                    staffsResponse.Data.ForEach(allStaff.Add);
                });
            });

            foreach (var staff in allStaff)
            {
                Console.WriteLine($"{staff.FirstName} {staff.LastSurname}");
            }
            Console.WriteLine("Press enter to continue...");
            Console.ReadLine();


            configuration = new Configuration()
            {
                AccessToken = token,
                BasePath = $"{baseUrl}identity/v2/2021"
            };

            var identitiesApi = new IdentitiesApi(configuration);
            var identitiesResponse = identitiesApi.IdentitiesSearch(new List<IdentitySearchRequest>()
            {
                new IdentitySearchRequest
                    {
                        BirthDate = new DateTime(2016, 3, 1),
                        FirstName = "Tom",
                        LastSurname = "Christensen",
                        EducationOrganizationId = 7533
                    }
            });

            Console.WriteLine($"Identity SearchKey: {identitiesResponse.SearchKey}");
            foreach (var searchResponse in identitiesResponse.SearchResponses)
            {
                foreach (var response in searchResponse.Responses)
                {
                    Console.WriteLine($"{response.FirstName} {response.LastSurname} - {response.Score}");
                }
            }
            Console.WriteLine("Press enter to continue...");
            Console.ReadLine();

            configuration = new Configuration()
            {
                AccessToken = token,
                BasePath = $"{baseUrl}immunizations/v1/2021"
            };

            var immunizationsApi = new ImmunizationsApi(configuration);
            var immunizationsResponse = immunizationsApi.ImmunizationsGetAsyncWithHttpInfo("1003734049");
            
            var patient = immunizationsResponse.Result.Data.Data;
            Console.WriteLine("Immunizations:");
            Console.WriteLine($"StudentUniqueId: {patient.StudentReference.StudentUniqueId}, EducationOrganizationId: {patient.StudentEducationOrganizationReference?.EducationOrganizationId}");

            foreach (var immunization in patient.PharmacyTreatmentAdministrations)
            {
                Console.WriteLine($"AdministeredDate: {immunization.AdministeredDate.Value.ToShortDateString()}, AdministeredCode: {immunization.AdministeredCode.Text}");
            }

            Console.WriteLine();
            Console.WriteLine("Press enter to continue...");
            Console.ReadLine();



            //configuration = new Configuration()
            //{
            //    AccessToken = tokenRetriever.ObtainNewBearerToken(),
            //    BasePath = $"{baseUrl}composites/v1/2020"
            //};
            //var schoolsApi = new Api.Enrollment.SchoolsApi(configuration);
            //var schoolsResponse = schoolsApi.GetSchoolsByLocalEducationAgencyAsync("7533");//228b703fdcb444f183fafe274bb2d664

            //var sectionsApi = new Api.Enrollment.SectionsApi(configuration);
            //var sectionsResponse = sectionsApi.GetSectionsByLocalEducationAgencyAsyncWithHttpInfo("228b703fdcb444f183fafe274bb2d664");

            //var staffApi = new Api.Enrollments.StaffsApi(configuration);
            //var staffResult = staffApi.GetStaffsByLocalEducationAgencyAsyncWithHttpInfo("228b703fdcb444f183fafe274bb2d664");

            //foreach (var staff in staffResult.Result.Data)
            //{
            //    Console.WriteLine($"{staff.FirstName} {staff.LastSurname}");
            //}
        }
    }
}
