/* 
 * Identity API Endpoints
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 2
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RestSharp;
using EdFi.OdsApi.Sdk.Client;
using EdFi.OdsApi.Sdk.Models.Identity;

namespace EdFi.OdsApi.Sdk.Api.Identity
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public interface IIdentitiesApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// Creates a new Unique Id for the given Identity information.
        /// </summary>
        /// <remarks>
        /// Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>string</returns>
        string IdentitiesCreate (IdentityCreateRequest request);

        /// <summary>
        /// Creates a new Unique Id for the given Identity information.
        /// </summary>
        /// <remarks>
        /// Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>ApiResponse of string</returns>
        ApiResponse<string> IdentitiesCreateWithHttpInfo (IdentityCreateRequest request);
        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>IdentitySearchResponse</returns>
        IdentitySearchResponse IdentitiesFind (List<string> uniqueIds);

        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>ApiResponse of IdentitySearchResponse</returns>
        ApiResponse<IdentitySearchResponse> IdentitiesFindWithHttpInfo (List<string> uniqueIds);
        /// <summary>
        /// Retrieve a single person record from their Unique Id.
        /// </summary>
        /// <remarks>
        /// Returns either a single identity or 404 and no data
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>IdentityResponse</returns>
        IdentityResponse IdentitiesGetById (string id);

        /// <summary>
        /// Retrieve a single person record from their Unique Id.
        /// </summary>
        /// <remarks>
        /// Returns either a single identity or 404 and no data
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>ApiResponse of IdentityResponse</returns>
        ApiResponse<IdentityResponse> IdentitiesGetByIdWithHttpInfo (string id);
        /// <summary>
        /// Retrieve asynchronous search results from a previously created request.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>IdentitySearchResponse</returns>
        IdentitySearchResponse IdentitiesResult (string id);

        /// <summary>
        /// Retrieve asynchronous search results from a previously created request.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>ApiResponse of IdentitySearchResponse</returns>
        ApiResponse<IdentitySearchResponse> IdentitiesResultWithHttpInfo (string id);
        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>IdentitySearchResponse</returns>
        IdentitySearchResponse IdentitiesSearch (List<IdentitySearchRequest> criteria);

        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>ApiResponse of IdentitySearchResponse</returns>
        ApiResponse<IdentitySearchResponse> IdentitiesSearchWithHttpInfo (List<IdentitySearchRequest> criteria);
        #endregion Synchronous Operations
        #region Asynchronous Operations
        /// <summary>
        /// Creates a new Unique Id for the given Identity information.
        /// </summary>
        /// <remarks>
        /// Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>Task of string</returns>
        System.Threading.Tasks.Task<string> IdentitiesCreateAsync (IdentityCreateRequest request);

        /// <summary>
        /// Creates a new Unique Id for the given Identity information.
        /// </summary>
        /// <remarks>
        /// Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>Task of ApiResponse (string)</returns>
        System.Threading.Tasks.Task<ApiResponse<string>> IdentitiesCreateAsyncWithHttpInfo (IdentityCreateRequest request);
        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>Task of IdentitySearchResponse</returns>
        System.Threading.Tasks.Task<IdentitySearchResponse> IdentitiesFindAsync (List<string> uniqueIds);

        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>Task of ApiResponse (IdentitySearchResponse)</returns>
        System.Threading.Tasks.Task<ApiResponse<IdentitySearchResponse>> IdentitiesFindAsyncWithHttpInfo (List<string> uniqueIds);
        /// <summary>
        /// Retrieve a single person record from their Unique Id.
        /// </summary>
        /// <remarks>
        /// Returns either a single identity or 404 and no data
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>Task of IdentityResponse</returns>
        System.Threading.Tasks.Task<IdentityResponse> IdentitiesGetByIdAsync (string id);

        /// <summary>
        /// Retrieve a single person record from their Unique Id.
        /// </summary>
        /// <remarks>
        /// Returns either a single identity or 404 and no data
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>Task of ApiResponse (IdentityResponse)</returns>
        System.Threading.Tasks.Task<ApiResponse<IdentityResponse>> IdentitiesGetByIdAsyncWithHttpInfo (string id);
        /// <summary>
        /// Retrieve asynchronous search results from a previously created request.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>Task of IdentitySearchResponse</returns>
        System.Threading.Tasks.Task<IdentitySearchResponse> IdentitiesResultAsync (string id);

        /// <summary>
        /// Retrieve asynchronous search results from a previously created request.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>Task of ApiResponse (IdentitySearchResponse)</returns>
        System.Threading.Tasks.Task<ApiResponse<IdentitySearchResponse>> IdentitiesResultAsyncWithHttpInfo (string id);
        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>Task of IdentitySearchResponse</returns>
        System.Threading.Tasks.Task<IdentitySearchResponse> IdentitiesSearchAsync (List<IdentitySearchRequest> criteria);

        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches.
        /// </summary>
        /// <remarks>
        /// 
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>Task of ApiResponse (IdentitySearchResponse)</returns>
        System.Threading.Tasks.Task<ApiResponse<IdentitySearchResponse>> IdentitiesSearchAsyncWithHttpInfo (List<IdentitySearchRequest> criteria);
        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public partial class IdentitiesApi : IIdentitiesApi
    {
        private EdFi.OdsApi.Sdk.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="IdentitiesApi"/> class.
        /// </summary>
        /// <returns></returns>
        public IdentitiesApi(String basePath)
        {
            this.Configuration = new Configuration { BasePath = basePath };

            ExceptionFactory = EdFi.OdsApi.Sdk.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="IdentitiesApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public IdentitiesApi(Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = EdFi.OdsApi.Sdk.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public Configuration Configuration {get; set;}

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public EdFi.OdsApi.Sdk.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public IDictionary<String, String> DefaultHeader()
        {
            return new ReadOnlyDictionary<string, string>(this.Configuration.DefaultHeader);
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        /// Creates a new Unique Id for the given Identity information. Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>string</returns>
        public string IdentitiesCreate (IdentityCreateRequest request)
        {
             ApiResponse<string> localVarResponse = IdentitiesCreateWithHttpInfo(request);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Creates a new Unique Id for the given Identity information. Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>ApiResponse of string</returns>
        public ApiResponse< string > IdentitiesCreateWithHttpInfo (IdentityCreateRequest request)
        {
            // verify the required parameter 'request' is set
            if (request == null)
                throw new ApiException(400, "Missing required parameter 'request' when calling IdentitiesApi->IdentitiesCreate");

            var localVarPath = "/identities";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json", 
                "text/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (request != null && request.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(request); // http body (model) parameter
            }
            else
            {
                localVarPostBody = request; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesCreate", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<string>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (string) Configuration.ApiClient.Deserialize(localVarResponse, typeof(string)));
        }

        /// <summary>
        /// Creates a new Unique Id for the given Identity information. Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>Task of string</returns>
        public async System.Threading.Tasks.Task<string> IdentitiesCreateAsync (IdentityCreateRequest request)
        {
             ApiResponse<string> localVarResponse = await IdentitiesCreateAsyncWithHttpInfo(request);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Creates a new Unique Id for the given Identity information. Assumption here is that the user has verified that possible matches are not correct matches. Returns the created identity information along with the assigned Unique Id.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="request">Identity object to be created.</param>
        /// <returns>Task of ApiResponse (string)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<string>> IdentitiesCreateAsyncWithHttpInfo (IdentityCreateRequest request)
        {
            // verify the required parameter 'request' is set
            if (request == null)
                throw new ApiException(400, "Missing required parameter 'request' when calling IdentitiesApi->IdentitiesCreate");

            var localVarPath = "/identities";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json", 
                "text/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (request != null && request.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(request); // http body (model) parameter
            }
            else
            {
                localVarPostBody = request; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesCreate", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<string>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (string) Configuration.ApiClient.Deserialize(localVarResponse, typeof(string)));
        }

        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>IdentitySearchResponse</returns>
        public IdentitySearchResponse IdentitiesFind (List<string> uniqueIds)
        {
             ApiResponse<IdentitySearchResponse> localVarResponse = IdentitiesFindWithHttpInfo(uniqueIds);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>ApiResponse of IdentitySearchResponse</returns>
        public ApiResponse< IdentitySearchResponse > IdentitiesFindWithHttpInfo (List<string> uniqueIds)
        {
            // verify the required parameter 'uniqueIds' is set
            if (uniqueIds == null)
                throw new ApiException(400, "Missing required parameter 'uniqueIds' when calling IdentitiesApi->IdentitiesFind");

            var localVarPath = "/identities/find";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json", 
                "text/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (uniqueIds != null && uniqueIds.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(uniqueIds); // http body (model) parameter
            }
            else
            {
                localVarPostBody = uniqueIds; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesFind", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentitySearchResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentitySearchResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentitySearchResponse)));
        }

        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>Task of IdentitySearchResponse</returns>
        public async System.Threading.Tasks.Task<IdentitySearchResponse> IdentitiesFindAsync (List<string> uniqueIds)
        {
             ApiResponse<IdentitySearchResponse> localVarResponse = await IdentitiesFindAsyncWithHttpInfo(uniqueIds);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Retrieve a multiple person records from their Unique Ids. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="uniqueIds">Unique Ids of the persons to be retrieved.</param>
        /// <returns>Task of ApiResponse (IdentitySearchResponse)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<IdentitySearchResponse>> IdentitiesFindAsyncWithHttpInfo (List<string> uniqueIds)
        {
            // verify the required parameter 'uniqueIds' is set
            if (uniqueIds == null)
                throw new ApiException(400, "Missing required parameter 'uniqueIds' when calling IdentitiesApi->IdentitiesFind");

            var localVarPath = "/identities/find";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json", 
                "text/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (uniqueIds != null && uniqueIds.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(uniqueIds); // http body (model) parameter
            }
            else
            {
                localVarPostBody = uniqueIds; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesFind", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentitySearchResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentitySearchResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentitySearchResponse)));
        }

        /// <summary>
        /// Retrieve a single person record from their Unique Id. Returns either a single identity or 404 and no data
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>IdentityResponse</returns>
        public IdentityResponse IdentitiesGetById (string id)
        {
             ApiResponse<IdentityResponse> localVarResponse = IdentitiesGetByIdWithHttpInfo(id);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Retrieve a single person record from their Unique Id. Returns either a single identity or 404 and no data
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>ApiResponse of IdentityResponse</returns>
        public ApiResponse< IdentityResponse > IdentitiesGetByIdWithHttpInfo (string id)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling IdentitiesApi->IdentitiesGetById");

            var localVarPath = "/identities/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesGetById", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentityResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentityResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentityResponse)));
        }

        /// <summary>
        /// Retrieve a single person record from their Unique Id. Returns either a single identity or 404 and no data
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>Task of IdentityResponse</returns>
        public async System.Threading.Tasks.Task<IdentityResponse> IdentitiesGetByIdAsync (string id)
        {
             ApiResponse<IdentityResponse> localVarResponse = await IdentitiesGetByIdAsyncWithHttpInfo(id);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Retrieve a single person record from their Unique Id. Returns either a single identity or 404 and no data
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">Unique Id of the identity to be retrieved</param>
        /// <returns>Task of ApiResponse (IdentityResponse)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<IdentityResponse>> IdentitiesGetByIdAsyncWithHttpInfo (string id)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling IdentitiesApi->IdentitiesGetById");

            var localVarPath = "/identities/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesGetById", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentityResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentityResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentityResponse)));
        }

        /// <summary>
        /// Retrieve asynchronous search results from a previously created request. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>IdentitySearchResponse</returns>
        public IdentitySearchResponse IdentitiesResult (string id)
        {
             ApiResponse<IdentitySearchResponse> localVarResponse = IdentitiesResultWithHttpInfo(id);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Retrieve asynchronous search results from a previously created request. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>ApiResponse of IdentitySearchResponse</returns>
        public ApiResponse< IdentitySearchResponse > IdentitiesResultWithHttpInfo (string id)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling IdentitiesApi->IdentitiesResult");

            var localVarPath = "/identities/results/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesResult", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentitySearchResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentitySearchResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentitySearchResponse)));
        }

        /// <summary>
        /// Retrieve asynchronous search results from a previously created request. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>Task of IdentitySearchResponse</returns>
        public async System.Threading.Tasks.Task<IdentitySearchResponse> IdentitiesResultAsync (string id)
        {
             ApiResponse<IdentitySearchResponse> localVarResponse = await IdentitiesResultAsyncWithHttpInfo(id);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Retrieve asynchronous search results from a previously created request. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">The search token provided by a Find or Search request.</param>
        /// <returns>Task of ApiResponse (IdentitySearchResponse)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<IdentitySearchResponse>> IdentitiesResultAsyncWithHttpInfo (string id)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling IdentitiesApi->IdentitiesResult");

            var localVarPath = "/identities/results/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesResult", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentitySearchResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentitySearchResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentitySearchResponse)));
        }

        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>IdentitySearchResponse</returns>
        public IdentitySearchResponse IdentitiesSearch (List<IdentitySearchRequest> criteria)
        {
             ApiResponse<IdentitySearchResponse> localVarResponse = IdentitiesSearchWithHttpInfo(criteria);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>ApiResponse of IdentitySearchResponse</returns>
        public ApiResponse< IdentitySearchResponse > IdentitiesSearchWithHttpInfo (List<IdentitySearchRequest> criteria)
        {
            // verify the required parameter 'criteria' is set
            if (criteria == null)
                throw new ApiException(400, "Missing required parameter 'criteria' when calling IdentitiesApi->IdentitiesSearch");

            var localVarPath = "/identities/search";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json", 
                "text/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (criteria != null && criteria.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(criteria); // http body (model) parameter
            }
            else
            {
                localVarPostBody = criteria; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesSearch", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentitySearchResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentitySearchResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentitySearchResponse)));
        }

        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>Task of IdentitySearchResponse</returns>
        public async System.Threading.Tasks.Task<IdentitySearchResponse> IdentitiesSearchAsync (List<IdentitySearchRequest> criteria)
        {
             ApiResponse<IdentitySearchResponse> localVarResponse = await IdentitiesSearchAsyncWithHttpInfo(criteria);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Lookup existing Unique Ids for a persons, or suggest possible matches. 
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="criteria">One or more identity search request objects.</param>
        /// <returns>Task of ApiResponse (IdentitySearchResponse)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<IdentitySearchResponse>> IdentitiesSearchAsyncWithHttpInfo (List<IdentitySearchRequest> criteria)
        {
            // verify the required parameter 'criteria' is set
            if (criteria == null)
                throw new ApiException(400, "Missing required parameter 'criteria' when calling IdentitiesApi->IdentitiesSearch");

            var localVarPath = "/identities/search";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json", 
                "text/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json",
                "text/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (criteria != null && criteria.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(criteria); // http body (model) parameter
            }
            else
            {
                localVarPostBody = criteria; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("IdentitiesSearch", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<IdentitySearchResponse>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (IdentitySearchResponse) Configuration.ApiClient.Deserialize(localVarResponse, typeof(IdentitySearchResponse)));
        }

    }
}
