/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Enrollment
{
    /// <summary>
    /// StaffStaffElectronicMail
    /// </summary>
    [DataContract]
    public partial class StaffStaffElectronicMail :  IEquatable<StaffStaffElectronicMail>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="StaffStaffElectronicMail" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected StaffStaffElectronicMail() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="StaffStaffElectronicMail" /> class.
        /// </summary>
        /// <param name="ElectronicMailTypeDescriptor">The type of email listed for an individual or organization. For example: Home/Personal, Work, etc.) (required).</param>
        /// <param name="ElectronicMailAddress">The electronic mail (e-mail) address listed for an individual or organization. (required).</param>
        public StaffStaffElectronicMail(string ElectronicMailTypeDescriptor = default(string), string ElectronicMailAddress = default(string))
        {
            // to ensure "ElectronicMailTypeDescriptor" is required (not null)
            if (ElectronicMailTypeDescriptor == null)
            {
                throw new InvalidDataException("ElectronicMailTypeDescriptor is a required property for StaffStaffElectronicMail and cannot be null");
            }
            else
            {
                this.ElectronicMailTypeDescriptor = ElectronicMailTypeDescriptor;
            }
            // to ensure "ElectronicMailAddress" is required (not null)
            if (ElectronicMailAddress == null)
            {
                throw new InvalidDataException("ElectronicMailAddress is a required property for StaffStaffElectronicMail and cannot be null");
            }
            else
            {
                this.ElectronicMailAddress = ElectronicMailAddress;
            }
        }
        
        /// <summary>
        /// The type of email listed for an individual or organization. For example: Home/Personal, Work, etc.)
        /// </summary>
        /// <value>The type of email listed for an individual or organization. For example: Home/Personal, Work, etc.)</value>
        [DataMember(Name="electronicMailTypeDescriptor", EmitDefaultValue=false)]
        public string ElectronicMailTypeDescriptor { get; set; }

        /// <summary>
        /// The electronic mail (e-mail) address listed for an individual or organization.
        /// </summary>
        /// <value>The electronic mail (e-mail) address listed for an individual or organization.</value>
        [DataMember(Name="electronicMailAddress", EmitDefaultValue=false)]
        public string ElectronicMailAddress { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class StaffStaffElectronicMail {\n");
            sb.Append("  ElectronicMailTypeDescriptor: ").Append(ElectronicMailTypeDescriptor).Append("\n");
            sb.Append("  ElectronicMailAddress: ").Append(ElectronicMailAddress).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as StaffStaffElectronicMail);
        }

        /// <summary>
        /// Returns true if StaffStaffElectronicMail instances are equal
        /// </summary>
        /// <param name="input">Instance of StaffStaffElectronicMail to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(StaffStaffElectronicMail input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.ElectronicMailTypeDescriptor == input.ElectronicMailTypeDescriptor ||
                    (this.ElectronicMailTypeDescriptor != null &&
                    this.ElectronicMailTypeDescriptor.Equals(input.ElectronicMailTypeDescriptor))
                ) && 
                (
                    this.ElectronicMailAddress == input.ElectronicMailAddress ||
                    (this.ElectronicMailAddress != null &&
                    this.ElectronicMailAddress.Equals(input.ElectronicMailAddress))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.ElectronicMailTypeDescriptor != null)
                    hashCode = hashCode * 59 + this.ElectronicMailTypeDescriptor.GetHashCode();
                if (this.ElectronicMailAddress != null)
                    hashCode = hashCode * 59 + this.ElectronicMailAddress.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // ElectronicMailTypeDescriptor (string) maxLength
            if(this.ElectronicMailTypeDescriptor != null && this.ElectronicMailTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ElectronicMailTypeDescriptor, length must be less than 306.", new [] { "ElectronicMailTypeDescriptor" });
            }

            // ElectronicMailAddress (string) maxLength
            if(this.ElectronicMailAddress != null && this.ElectronicMailAddress.Length > 128)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ElectronicMailAddress, length must be less than 128.", new [] { "ElectronicMailAddress" });
            }

            yield break;
        }
    }

}
