/* 
 * Identity API Endpoints
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 2
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Identity
{
    /// <summary>
    /// Local
    /// </summary>
    [DataContract]
    public partial class Local :  IEquatable<Local>, IValidatableObject
    {
        /// <summary>
        /// Either Student or Teacher
        /// </summary>
        /// <value>Either Student or Teacher</value>
        [JsonConverter(typeof(StringEnumConverter))]
        public enum TypeDescriptionEnum
        {
            
            /// <summary>
            /// Enum Staff for "Staff"
            /// </summary>
            [EnumMember(Value = "Staff")]
            Staff = 1,
            
            /// <summary>
            /// Enum Student for "Student"
            /// </summary>
            [EnumMember(Value = "Student")]
            Student = 2
        }

        /// <summary>
        /// Either Student or Teacher
        /// </summary>
        /// <value>Either Student or Teacher</value>
        [DataMember(Name="TypeDescription", EmitDefaultValue=false)]
        public TypeDescriptionEnum? TypeDescription { get; set; }
        /// <summary>
        /// Initializes a new instance of the <see cref="Local" /> class.
        /// </summary>
        /// <param name="EducationOrganizationId">EducationOrganizationId.</param>
        /// <param name="LocalPersonId">A string uniquely identifying the student for the Vendor Software.</param>
        /// <param name="Description">A description of the person local row key - ex. Manually created wise id.</param>
        /// <param name="Email">Email.</param>
        /// <param name="TypeDescription">Either Student or Teacher.</param>
        /// <param name="WiseSecureRoleDescription">A federation role Ex Student or Teacher.</param>
        public Local(int? EducationOrganizationId = default(int?), string LocalPersonId = default(string), string Description = default(string), string Email = default(string), TypeDescriptionEnum? TypeDescription = default(TypeDescriptionEnum?), string WiseSecureRoleDescription = default(string))
        {
            this.EducationOrganizationId = EducationOrganizationId;
            this.LocalPersonId = LocalPersonId;
            this.Description = Description;
            this.Email = Email;
            this.TypeDescription = TypeDescription;
            this.WiseSecureRoleDescription = WiseSecureRoleDescription;
        }
        
        /// <summary>
        /// Gets or Sets EducationOrganizationId
        /// </summary>
        [DataMember(Name="EducationOrganizationId", EmitDefaultValue=false)]
        public int? EducationOrganizationId { get; set; }

        /// <summary>
        /// A string uniquely identifying the student for the Vendor Software
        /// </summary>
        /// <value>A string uniquely identifying the student for the Vendor Software</value>
        [DataMember(Name="LocalPersonId", EmitDefaultValue=false)]
        public string LocalPersonId { get; set; }

        /// <summary>
        /// A description of the person local row key - ex. Manually created wise id
        /// </summary>
        /// <value>A description of the person local row key - ex. Manually created wise id</value>
        [DataMember(Name="Description", EmitDefaultValue=false)]
        public string Description { get; set; }

        /// <summary>
        /// Gets or Sets Email
        /// </summary>
        [DataMember(Name="Email", EmitDefaultValue=false)]
        public string Email { get; set; }


        /// <summary>
        /// A federation role Ex Student or Teacher
        /// </summary>
        /// <value>A federation role Ex Student or Teacher</value>
        [DataMember(Name="WiseSecureRoleDescription", EmitDefaultValue=false)]
        public string WiseSecureRoleDescription { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class Local {\n");
            sb.Append("  EducationOrganizationId: ").Append(EducationOrganizationId).Append("\n");
            sb.Append("  LocalPersonId: ").Append(LocalPersonId).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  Email: ").Append(Email).Append("\n");
            sb.Append("  TypeDescription: ").Append(TypeDescription).Append("\n");
            sb.Append("  WiseSecureRoleDescription: ").Append(WiseSecureRoleDescription).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as Local);
        }

        /// <summary>
        /// Returns true if Local instances are equal
        /// </summary>
        /// <param name="input">Instance of Local to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(Local input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.EducationOrganizationId == input.EducationOrganizationId ||
                    (this.EducationOrganizationId != null &&
                    this.EducationOrganizationId.Equals(input.EducationOrganizationId))
                ) && 
                (
                    this.LocalPersonId == input.LocalPersonId ||
                    (this.LocalPersonId != null &&
                    this.LocalPersonId.Equals(input.LocalPersonId))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.Email == input.Email ||
                    (this.Email != null &&
                    this.Email.Equals(input.Email))
                ) && 
                (
                    this.TypeDescription == input.TypeDescription ||
                    (this.TypeDescription != null &&
                    this.TypeDescription.Equals(input.TypeDescription))
                ) && 
                (
                    this.WiseSecureRoleDescription == input.WiseSecureRoleDescription ||
                    (this.WiseSecureRoleDescription != null &&
                    this.WiseSecureRoleDescription.Equals(input.WiseSecureRoleDescription))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.EducationOrganizationId != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationId.GetHashCode();
                if (this.LocalPersonId != null)
                    hashCode = hashCode * 59 + this.LocalPersonId.GetHashCode();
                if (this.Description != null)
                    hashCode = hashCode * 59 + this.Description.GetHashCode();
                if (this.Email != null)
                    hashCode = hashCode * 59 + this.Email.GetHashCode();
                if (this.TypeDescription != null)
                    hashCode = hashCode * 59 + this.TypeDescription.GetHashCode();
                if (this.WiseSecureRoleDescription != null)
                    hashCode = hashCode * 59 + this.WiseSecureRoleDescription.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
