/* 
 * Immunizations API Endpoints
 *
 * Coming soon(ish)! The WI Department of Public Instruction in conjunction with the WI Department of Health Services is working to facilitate the retrieval of HL7 immunization data from the WI Immunization Registry via the EdFi API.
 *
 * OpenAPI spec version: 1
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Immunizations
{
    /// <summary>
    /// The student identifiers and the student immunization history.
    /// </summary>
    [DataContract]
    public partial class ImmunizationsStudentHistory :  IEquatable<ImmunizationsStudentHistory>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ImmunizationsStudentHistory" /> class.
        /// </summary>
        /// <param name="PatientIdentifier">This is the WiR internal identifier which could be useful when trying to cross reference the WiR (ex. updating student information in the WiR to help resolve MultipleMatches or NotFound).</param>
        /// <param name="StudentReference">A reference to the student to help confirm the correct EdFi student is being matched to the correct WiR patient.</param>
        /// <param name="StudentEducationOrganizationReference">A reference to the studentEducationOrganizationAssociation to help confirm the correct EdFi student demographics are being matched to the correct WiR patient.</param>
        /// <param name="PharmacyTreatmentAdministrations">An array of the student immunization history.</param>
        public ImmunizationsStudentHistory(ImmunizationsPatientIdentifier PatientIdentifier = default(ImmunizationsPatientIdentifier), EdFiStudentReference StudentReference = default(EdFiStudentReference), EdFiStudentEducationOrganizationAssociationReference StudentEducationOrganizationReference = default(EdFiStudentEducationOrganizationAssociationReference), List<ImmunizationsPharmacyTreatmentAdministration> PharmacyTreatmentAdministrations = default(List<ImmunizationsPharmacyTreatmentAdministration>))
        {
            this.PatientIdentifier = PatientIdentifier;
            this.StudentReference = StudentReference;
            this.StudentEducationOrganizationReference = StudentEducationOrganizationReference;
            this.PharmacyTreatmentAdministrations = PharmacyTreatmentAdministrations;
        }
        
        /// <summary>
        /// This is the WiR internal identifier which could be useful when trying to cross reference the WiR (ex. updating student information in the WiR to help resolve MultipleMatches or NotFound)
        /// </summary>
        /// <value>This is the WiR internal identifier which could be useful when trying to cross reference the WiR (ex. updating student information in the WiR to help resolve MultipleMatches or NotFound)</value>
        [DataMember(Name="patientIdentifier", EmitDefaultValue=false)]
        public ImmunizationsPatientIdentifier PatientIdentifier { get; set; }

        /// <summary>
        /// A reference to the student to help confirm the correct EdFi student is being matched to the correct WiR patient
        /// </summary>
        /// <value>A reference to the student to help confirm the correct EdFi student is being matched to the correct WiR patient</value>
        [DataMember(Name="studentReference", EmitDefaultValue=false)]
        public EdFiStudentReference StudentReference { get; set; }

        /// <summary>
        /// A reference to the studentEducationOrganizationAssociation to help confirm the correct EdFi student demographics are being matched to the correct WiR patient
        /// </summary>
        /// <value>A reference to the studentEducationOrganizationAssociation to help confirm the correct EdFi student demographics are being matched to the correct WiR patient</value>
        [DataMember(Name="studentEducationOrganizationReference", EmitDefaultValue=false)]
        public EdFiStudentEducationOrganizationAssociationReference StudentEducationOrganizationReference { get; set; }

        /// <summary>
        /// An array of the student immunization history
        /// </summary>
        /// <value>An array of the student immunization history</value>
        [DataMember(Name="PharmacyTreatmentAdministrations", EmitDefaultValue=false)]
        public List<ImmunizationsPharmacyTreatmentAdministration> PharmacyTreatmentAdministrations { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class ImmunizationsStudentHistory {\n");
            sb.Append("  PatientIdentifier: ").Append(PatientIdentifier).Append("\n");
            sb.Append("  StudentReference: ").Append(StudentReference).Append("\n");
            sb.Append("  StudentEducationOrganizationReference: ").Append(StudentEducationOrganizationReference).Append("\n");
            sb.Append("  PharmacyTreatmentAdministrations: ").Append(PharmacyTreatmentAdministrations).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as ImmunizationsStudentHistory);
        }

        /// <summary>
        /// Returns true if ImmunizationsStudentHistory instances are equal
        /// </summary>
        /// <param name="input">Instance of ImmunizationsStudentHistory to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(ImmunizationsStudentHistory input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.PatientIdentifier == input.PatientIdentifier ||
                    (this.PatientIdentifier != null &&
                    this.PatientIdentifier.Equals(input.PatientIdentifier))
                ) && 
                (
                    this.StudentReference == input.StudentReference ||
                    (this.StudentReference != null &&
                    this.StudentReference.Equals(input.StudentReference))
                ) && 
                (
                    this.StudentEducationOrganizationReference == input.StudentEducationOrganizationReference ||
                    (this.StudentEducationOrganizationReference != null &&
                    this.StudentEducationOrganizationReference.Equals(input.StudentEducationOrganizationReference))
                ) && 
                (
                    this.PharmacyTreatmentAdministrations == input.PharmacyTreatmentAdministrations ||
                    this.PharmacyTreatmentAdministrations != null &&
                    this.PharmacyTreatmentAdministrations.SequenceEqual(input.PharmacyTreatmentAdministrations)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.PatientIdentifier != null)
                    hashCode = hashCode * 59 + this.PatientIdentifier.GetHashCode();
                if (this.StudentReference != null)
                    hashCode = hashCode * 59 + this.StudentReference.GetHashCode();
                if (this.StudentEducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.StudentEducationOrganizationReference.GetHashCode();
                if (this.PharmacyTreatmentAdministrations != null)
                    hashCode = hashCode * 59 + this.PharmacyTreatmentAdministrations.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
