/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiAcademicWeek
    /// </summary>
    [DataContract]
    public partial class EdFiAcademicWeek :  IEquatable<EdFiAcademicWeek>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiAcademicWeek" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiAcademicWeek() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiAcademicWeek" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="WeekIdentifier">The school label for the week. (required).</param>
        /// <param name="SchoolReference">SchoolReference (required).</param>
        /// <param name="BeginDate">The start date for the academic week. (required).</param>
        /// <param name="EndDate">The end date for the academic week. (required).</param>
        /// <param name="TotalInstructionalDays">The total instructional days during the academic week. (required).</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiAcademicWeek(string Id = default(string), string WeekIdentifier = default(string), EdFiSchoolReference SchoolReference = default(EdFiSchoolReference), DateTime? BeginDate = default(DateTime?), DateTime? EndDate = default(DateTime?), int? TotalInstructionalDays = default(int?), string Etag = default(string))
        {
            // to ensure "WeekIdentifier" is required (not null)
            if (WeekIdentifier == null)
            {
                throw new InvalidDataException("WeekIdentifier is a required property for EdFiAcademicWeek and cannot be null");
            }
            else
            {
                this.WeekIdentifier = WeekIdentifier;
            }
            // to ensure "SchoolReference" is required (not null)
            if (SchoolReference == null)
            {
                throw new InvalidDataException("SchoolReference is a required property for EdFiAcademicWeek and cannot be null");
            }
            else
            {
                this.SchoolReference = SchoolReference;
            }
            // to ensure "BeginDate" is required (not null)
            if (BeginDate == null)
            {
                throw new InvalidDataException("BeginDate is a required property for EdFiAcademicWeek and cannot be null");
            }
            else
            {
                this.BeginDate = BeginDate;
            }
            // to ensure "EndDate" is required (not null)
            if (EndDate == null)
            {
                throw new InvalidDataException("EndDate is a required property for EdFiAcademicWeek and cannot be null");
            }
            else
            {
                this.EndDate = EndDate;
            }
            // to ensure "TotalInstructionalDays" is required (not null)
            if (TotalInstructionalDays == null)
            {
                throw new InvalidDataException("TotalInstructionalDays is a required property for EdFiAcademicWeek and cannot be null");
            }
            else
            {
                this.TotalInstructionalDays = TotalInstructionalDays;
            }
            this.Id = Id;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The school label for the week.
        /// </summary>
        /// <value>The school label for the week.</value>
        [DataMember(Name="weekIdentifier", EmitDefaultValue=false)]
        public string WeekIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets SchoolReference
        /// </summary>
        [DataMember(Name="schoolReference", EmitDefaultValue=false)]
        public EdFiSchoolReference SchoolReference { get; set; }

        /// <summary>
        /// The start date for the academic week.
        /// </summary>
        /// <value>The start date for the academic week.</value>
        [DataMember(Name="beginDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? BeginDate { get; set; }

        /// <summary>
        /// The end date for the academic week.
        /// </summary>
        /// <value>The end date for the academic week.</value>
        [DataMember(Name="endDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? EndDate { get; set; }

        /// <summary>
        /// The total instructional days during the academic week.
        /// </summary>
        /// <value>The total instructional days during the academic week.</value>
        [DataMember(Name="totalInstructionalDays", EmitDefaultValue=false)]
        public int? TotalInstructionalDays { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiAcademicWeek {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  WeekIdentifier: ").Append(WeekIdentifier).Append("\n");
            sb.Append("  SchoolReference: ").Append(SchoolReference).Append("\n");
            sb.Append("  BeginDate: ").Append(BeginDate).Append("\n");
            sb.Append("  EndDate: ").Append(EndDate).Append("\n");
            sb.Append("  TotalInstructionalDays: ").Append(TotalInstructionalDays).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiAcademicWeek);
        }

        /// <summary>
        /// Returns true if EdFiAcademicWeek instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiAcademicWeek to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiAcademicWeek input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.WeekIdentifier == input.WeekIdentifier ||
                    (this.WeekIdentifier != null &&
                    this.WeekIdentifier.Equals(input.WeekIdentifier))
                ) && 
                (
                    this.SchoolReference == input.SchoolReference ||
                    (this.SchoolReference != null &&
                    this.SchoolReference.Equals(input.SchoolReference))
                ) && 
                (
                    this.BeginDate == input.BeginDate ||
                    (this.BeginDate != null &&
                    this.BeginDate.Equals(input.BeginDate))
                ) && 
                (
                    this.EndDate == input.EndDate ||
                    (this.EndDate != null &&
                    this.EndDate.Equals(input.EndDate))
                ) && 
                (
                    this.TotalInstructionalDays == input.TotalInstructionalDays ||
                    (this.TotalInstructionalDays != null &&
                    this.TotalInstructionalDays.Equals(input.TotalInstructionalDays))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.WeekIdentifier != null)
                    hashCode = hashCode * 59 + this.WeekIdentifier.GetHashCode();
                if (this.SchoolReference != null)
                    hashCode = hashCode * 59 + this.SchoolReference.GetHashCode();
                if (this.BeginDate != null)
                    hashCode = hashCode * 59 + this.BeginDate.GetHashCode();
                if (this.EndDate != null)
                    hashCode = hashCode * 59 + this.EndDate.GetHashCode();
                if (this.TotalInstructionalDays != null)
                    hashCode = hashCode * 59 + this.TotalInstructionalDays.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // WeekIdentifier (string) maxLength
            if(this.WeekIdentifier != null && this.WeekIdentifier.Length > 80)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for WeekIdentifier, length must be less than 80.", new [] { "WeekIdentifier" });
            }

            yield break;
        }
    }

}
