/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiAccountCode
    /// </summary>
    [DataContract]
    public partial class EdFiAccountCode :  IEquatable<EdFiAccountCode>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiAccountCode" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiAccountCode() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiAccountCode" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="AccountClassificationDescriptor">The type of account code associated with the account. (required).</param>
        /// <param name="AccountCodeNumber">An account code defined for the education accounting system by the education organization. (required).</param>
        /// <param name="FiscalYear">The financial accounting year. (required).</param>
        /// <param name="EducationOrganizationReference">EducationOrganizationReference (required).</param>
        /// <param name="AccountCodeDescription">A description of the account code..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiAccountCode(string Id = default(string), string AccountClassificationDescriptor = default(string), string AccountCodeNumber = default(string), int? FiscalYear = default(int?), EdFiEducationOrganizationReference EducationOrganizationReference = default(EdFiEducationOrganizationReference), string AccountCodeDescription = default(string), string Etag = default(string))
        {
            // to ensure "AccountClassificationDescriptor" is required (not null)
            if (AccountClassificationDescriptor == null)
            {
                throw new InvalidDataException("AccountClassificationDescriptor is a required property for EdFiAccountCode and cannot be null");
            }
            else
            {
                this.AccountClassificationDescriptor = AccountClassificationDescriptor;
            }
            // to ensure "AccountCodeNumber" is required (not null)
            if (AccountCodeNumber == null)
            {
                throw new InvalidDataException("AccountCodeNumber is a required property for EdFiAccountCode and cannot be null");
            }
            else
            {
                this.AccountCodeNumber = AccountCodeNumber;
            }
            // to ensure "FiscalYear" is required (not null)
            if (FiscalYear == null)
            {
                throw new InvalidDataException("FiscalYear is a required property for EdFiAccountCode and cannot be null");
            }
            else
            {
                this.FiscalYear = FiscalYear;
            }
            // to ensure "EducationOrganizationReference" is required (not null)
            if (EducationOrganizationReference == null)
            {
                throw new InvalidDataException("EducationOrganizationReference is a required property for EdFiAccountCode and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = EducationOrganizationReference;
            }
            this.Id = Id;
            this.AccountCodeDescription = AccountCodeDescription;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The type of account code associated with the account.
        /// </summary>
        /// <value>The type of account code associated with the account.</value>
        [DataMember(Name="accountClassificationDescriptor", EmitDefaultValue=false)]
        public string AccountClassificationDescriptor { get; set; }

        /// <summary>
        /// An account code defined for the education accounting system by the education organization.
        /// </summary>
        /// <value>An account code defined for the education accounting system by the education organization.</value>
        [DataMember(Name="accountCodeNumber", EmitDefaultValue=false)]
        public string AccountCodeNumber { get; set; }

        /// <summary>
        /// The financial accounting year.
        /// </summary>
        /// <value>The financial accounting year.</value>
        [DataMember(Name="fiscalYear", EmitDefaultValue=false)]
        public int? FiscalYear { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// A description of the account code.
        /// </summary>
        /// <value>A description of the account code.</value>
        [DataMember(Name="accountCodeDescription", EmitDefaultValue=false)]
        public string AccountCodeDescription { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiAccountCode {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AccountClassificationDescriptor: ").Append(AccountClassificationDescriptor).Append("\n");
            sb.Append("  AccountCodeNumber: ").Append(AccountCodeNumber).Append("\n");
            sb.Append("  FiscalYear: ").Append(FiscalYear).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  AccountCodeDescription: ").Append(AccountCodeDescription).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiAccountCode);
        }

        /// <summary>
        /// Returns true if EdFiAccountCode instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiAccountCode to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiAccountCode input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AccountClassificationDescriptor == input.AccountClassificationDescriptor ||
                    (this.AccountClassificationDescriptor != null &&
                    this.AccountClassificationDescriptor.Equals(input.AccountClassificationDescriptor))
                ) && 
                (
                    this.AccountCodeNumber == input.AccountCodeNumber ||
                    (this.AccountCodeNumber != null &&
                    this.AccountCodeNumber.Equals(input.AccountCodeNumber))
                ) && 
                (
                    this.FiscalYear == input.FiscalYear ||
                    (this.FiscalYear != null &&
                    this.FiscalYear.Equals(input.FiscalYear))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.AccountCodeDescription == input.AccountCodeDescription ||
                    (this.AccountCodeDescription != null &&
                    this.AccountCodeDescription.Equals(input.AccountCodeDescription))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.AccountClassificationDescriptor != null)
                    hashCode = hashCode * 59 + this.AccountClassificationDescriptor.GetHashCode();
                if (this.AccountCodeNumber != null)
                    hashCode = hashCode * 59 + this.AccountCodeNumber.GetHashCode();
                if (this.FiscalYear != null)
                    hashCode = hashCode * 59 + this.FiscalYear.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.AccountCodeDescription != null)
                    hashCode = hashCode * 59 + this.AccountCodeDescription.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // AccountClassificationDescriptor (string) maxLength
            if(this.AccountClassificationDescriptor != null && this.AccountClassificationDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountClassificationDescriptor, length must be less than 306.", new [] { "AccountClassificationDescriptor" });
            }

            // AccountCodeNumber (string) maxLength
            if(this.AccountCodeNumber != null && this.AccountCodeNumber.Length > 50)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountCodeNumber, length must be less than 50.", new [] { "AccountCodeNumber" });
            }

            // AccountCodeDescription (string) maxLength
            if(this.AccountCodeDescription != null && this.AccountCodeDescription.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountCodeDescription, length must be less than 1024.", new [] { "AccountCodeDescription" });
            }

            yield break;
        }
    }

}
