/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiDisciplineIncidentExternalParticipant
    /// </summary>
    [DataContract]
    public partial class EdFiDisciplineIncidentExternalParticipant :  IEquatable<EdFiDisciplineIncidentExternalParticipant>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiDisciplineIncidentExternalParticipant" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiDisciplineIncidentExternalParticipant() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiDisciplineIncidentExternalParticipant" /> class.
        /// </summary>
        /// <param name="DisciplineIncidentParticipationCodeDescriptor">The role or type of participation of a student in a discipline incident; for example: Victim, Perpetrator, Witness, Reporter. (required).</param>
        /// <param name="FirstName">A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change. (required).</param>
        /// <param name="LastSurname">The name borne in common by members of a family. (required).</param>
        public EdFiDisciplineIncidentExternalParticipant(string DisciplineIncidentParticipationCodeDescriptor = default(string), string FirstName = default(string), string LastSurname = default(string))
        {
            // to ensure "DisciplineIncidentParticipationCodeDescriptor" is required (not null)
            if (DisciplineIncidentParticipationCodeDescriptor == null)
            {
                throw new InvalidDataException("DisciplineIncidentParticipationCodeDescriptor is a required property for EdFiDisciplineIncidentExternalParticipant and cannot be null");
            }
            else
            {
                this.DisciplineIncidentParticipationCodeDescriptor = DisciplineIncidentParticipationCodeDescriptor;
            }
            // to ensure "FirstName" is required (not null)
            if (FirstName == null)
            {
                throw new InvalidDataException("FirstName is a required property for EdFiDisciplineIncidentExternalParticipant and cannot be null");
            }
            else
            {
                this.FirstName = FirstName;
            }
            // to ensure "LastSurname" is required (not null)
            if (LastSurname == null)
            {
                throw new InvalidDataException("LastSurname is a required property for EdFiDisciplineIncidentExternalParticipant and cannot be null");
            }
            else
            {
                this.LastSurname = LastSurname;
            }
        }
        
        /// <summary>
        /// The role or type of participation of a student in a discipline incident; for example: Victim, Perpetrator, Witness, Reporter.
        /// </summary>
        /// <value>The role or type of participation of a student in a discipline incident; for example: Victim, Perpetrator, Witness, Reporter.</value>
        [DataMember(Name="disciplineIncidentParticipationCodeDescriptor", EmitDefaultValue=false)]
        public string DisciplineIncidentParticipationCodeDescriptor { get; set; }

        /// <summary>
        /// A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.
        /// </summary>
        /// <value>A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.</value>
        [DataMember(Name="firstName", EmitDefaultValue=false)]
        public string FirstName { get; set; }

        /// <summary>
        /// The name borne in common by members of a family.
        /// </summary>
        /// <value>The name borne in common by members of a family.</value>
        [DataMember(Name="lastSurname", EmitDefaultValue=false)]
        public string LastSurname { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiDisciplineIncidentExternalParticipant {\n");
            sb.Append("  DisciplineIncidentParticipationCodeDescriptor: ").Append(DisciplineIncidentParticipationCodeDescriptor).Append("\n");
            sb.Append("  FirstName: ").Append(FirstName).Append("\n");
            sb.Append("  LastSurname: ").Append(LastSurname).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiDisciplineIncidentExternalParticipant);
        }

        /// <summary>
        /// Returns true if EdFiDisciplineIncidentExternalParticipant instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiDisciplineIncidentExternalParticipant to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiDisciplineIncidentExternalParticipant input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.DisciplineIncidentParticipationCodeDescriptor == input.DisciplineIncidentParticipationCodeDescriptor ||
                    (this.DisciplineIncidentParticipationCodeDescriptor != null &&
                    this.DisciplineIncidentParticipationCodeDescriptor.Equals(input.DisciplineIncidentParticipationCodeDescriptor))
                ) && 
                (
                    this.FirstName == input.FirstName ||
                    (this.FirstName != null &&
                    this.FirstName.Equals(input.FirstName))
                ) && 
                (
                    this.LastSurname == input.LastSurname ||
                    (this.LastSurname != null &&
                    this.LastSurname.Equals(input.LastSurname))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.DisciplineIncidentParticipationCodeDescriptor != null)
                    hashCode = hashCode * 59 + this.DisciplineIncidentParticipationCodeDescriptor.GetHashCode();
                if (this.FirstName != null)
                    hashCode = hashCode * 59 + this.FirstName.GetHashCode();
                if (this.LastSurname != null)
                    hashCode = hashCode * 59 + this.LastSurname.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // DisciplineIncidentParticipationCodeDescriptor (string) maxLength
            if(this.DisciplineIncidentParticipationCodeDescriptor != null && this.DisciplineIncidentParticipationCodeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DisciplineIncidentParticipationCodeDescriptor, length must be less than 306.", new [] { "DisciplineIncidentParticipationCodeDescriptor" });
            }

            // FirstName (string) maxLength
            if(this.FirstName != null && this.FirstName.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for FirstName, length must be less than 75.", new [] { "FirstName" });
            }

            // LastSurname (string) maxLength
            if(this.LastSurname != null && this.LastSurname.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for LastSurname, length must be less than 75.", new [] { "LastSurname" });
            }

            yield break;
        }
    }

}
