/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiGradingPeriod
    /// </summary>
    [DataContract]
    public partial class EdFiGradingPeriod :  IEquatable<EdFiGradingPeriod>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiGradingPeriod" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiGradingPeriod() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiGradingPeriod" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="GradingPeriodDescriptor">The name of the period for which grades are reported. (required).</param>
        /// <param name="PeriodSequence">The sequential order of this period relative to other periods. (required).</param>
        /// <param name="SchoolReference">SchoolReference (required).</param>
        /// <param name="SchoolYearTypeReference">SchoolYearTypeReference (required).</param>
        /// <param name="BeginDate">Month, day, and year of the first day of the GradingPeriod. (required).</param>
        /// <param name="EndDate">Month, day, and year of the last day of the GradingPeriod. (required).</param>
        /// <param name="TotalInstructionalDays">Total days available for educational instruction during the GradingPeriod. (required).</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiGradingPeriod(string Id = default(string), string GradingPeriodDescriptor = default(string), int? PeriodSequence = default(int?), EdFiSchoolReference SchoolReference = default(EdFiSchoolReference), EdFiSchoolYearTypeReference SchoolYearTypeReference = default(EdFiSchoolYearTypeReference), DateTime? BeginDate = default(DateTime?), DateTime? EndDate = default(DateTime?), int? TotalInstructionalDays = default(int?), string Etag = default(string))
        {
            // to ensure "GradingPeriodDescriptor" is required (not null)
            if (GradingPeriodDescriptor == null)
            {
                throw new InvalidDataException("GradingPeriodDescriptor is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.GradingPeriodDescriptor = GradingPeriodDescriptor;
            }
            // to ensure "PeriodSequence" is required (not null)
            if (PeriodSequence == null)
            {
                throw new InvalidDataException("PeriodSequence is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.PeriodSequence = PeriodSequence;
            }
            // to ensure "SchoolReference" is required (not null)
            if (SchoolReference == null)
            {
                throw new InvalidDataException("SchoolReference is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.SchoolReference = SchoolReference;
            }
            // to ensure "SchoolYearTypeReference" is required (not null)
            if (SchoolYearTypeReference == null)
            {
                throw new InvalidDataException("SchoolYearTypeReference is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.SchoolYearTypeReference = SchoolYearTypeReference;
            }
            // to ensure "BeginDate" is required (not null)
            if (BeginDate == null)
            {
                throw new InvalidDataException("BeginDate is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.BeginDate = BeginDate;
            }
            // to ensure "EndDate" is required (not null)
            if (EndDate == null)
            {
                throw new InvalidDataException("EndDate is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.EndDate = EndDate;
            }
            // to ensure "TotalInstructionalDays" is required (not null)
            if (TotalInstructionalDays == null)
            {
                throw new InvalidDataException("TotalInstructionalDays is a required property for EdFiGradingPeriod and cannot be null");
            }
            else
            {
                this.TotalInstructionalDays = TotalInstructionalDays;
            }
            this.Id = Id;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The name of the period for which grades are reported.
        /// </summary>
        /// <value>The name of the period for which grades are reported.</value>
        [DataMember(Name="gradingPeriodDescriptor", EmitDefaultValue=false)]
        public string GradingPeriodDescriptor { get; set; }

        /// <summary>
        /// The sequential order of this period relative to other periods.
        /// </summary>
        /// <value>The sequential order of this period relative to other periods.</value>
        [DataMember(Name="periodSequence", EmitDefaultValue=false)]
        public int? PeriodSequence { get; set; }

        /// <summary>
        /// Gets or Sets SchoolReference
        /// </summary>
        [DataMember(Name="schoolReference", EmitDefaultValue=false)]
        public EdFiSchoolReference SchoolReference { get; set; }

        /// <summary>
        /// Gets or Sets SchoolYearTypeReference
        /// </summary>
        [DataMember(Name="schoolYearTypeReference", EmitDefaultValue=false)]
        public EdFiSchoolYearTypeReference SchoolYearTypeReference { get; set; }

        /// <summary>
        /// Month, day, and year of the first day of the GradingPeriod.
        /// </summary>
        /// <value>Month, day, and year of the first day of the GradingPeriod.</value>
        [DataMember(Name="beginDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? BeginDate { get; set; }

        /// <summary>
        /// Month, day, and year of the last day of the GradingPeriod.
        /// </summary>
        /// <value>Month, day, and year of the last day of the GradingPeriod.</value>
        [DataMember(Name="endDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? EndDate { get; set; }

        /// <summary>
        /// Total days available for educational instruction during the GradingPeriod.
        /// </summary>
        /// <value>Total days available for educational instruction during the GradingPeriod.</value>
        [DataMember(Name="totalInstructionalDays", EmitDefaultValue=false)]
        public int? TotalInstructionalDays { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiGradingPeriod {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  GradingPeriodDescriptor: ").Append(GradingPeriodDescriptor).Append("\n");
            sb.Append("  PeriodSequence: ").Append(PeriodSequence).Append("\n");
            sb.Append("  SchoolReference: ").Append(SchoolReference).Append("\n");
            sb.Append("  SchoolYearTypeReference: ").Append(SchoolYearTypeReference).Append("\n");
            sb.Append("  BeginDate: ").Append(BeginDate).Append("\n");
            sb.Append("  EndDate: ").Append(EndDate).Append("\n");
            sb.Append("  TotalInstructionalDays: ").Append(TotalInstructionalDays).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiGradingPeriod);
        }

        /// <summary>
        /// Returns true if EdFiGradingPeriod instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiGradingPeriod to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiGradingPeriod input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.GradingPeriodDescriptor == input.GradingPeriodDescriptor ||
                    (this.GradingPeriodDescriptor != null &&
                    this.GradingPeriodDescriptor.Equals(input.GradingPeriodDescriptor))
                ) && 
                (
                    this.PeriodSequence == input.PeriodSequence ||
                    (this.PeriodSequence != null &&
                    this.PeriodSequence.Equals(input.PeriodSequence))
                ) && 
                (
                    this.SchoolReference == input.SchoolReference ||
                    (this.SchoolReference != null &&
                    this.SchoolReference.Equals(input.SchoolReference))
                ) && 
                (
                    this.SchoolYearTypeReference == input.SchoolYearTypeReference ||
                    (this.SchoolYearTypeReference != null &&
                    this.SchoolYearTypeReference.Equals(input.SchoolYearTypeReference))
                ) && 
                (
                    this.BeginDate == input.BeginDate ||
                    (this.BeginDate != null &&
                    this.BeginDate.Equals(input.BeginDate))
                ) && 
                (
                    this.EndDate == input.EndDate ||
                    (this.EndDate != null &&
                    this.EndDate.Equals(input.EndDate))
                ) && 
                (
                    this.TotalInstructionalDays == input.TotalInstructionalDays ||
                    (this.TotalInstructionalDays != null &&
                    this.TotalInstructionalDays.Equals(input.TotalInstructionalDays))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.GradingPeriodDescriptor != null)
                    hashCode = hashCode * 59 + this.GradingPeriodDescriptor.GetHashCode();
                if (this.PeriodSequence != null)
                    hashCode = hashCode * 59 + this.PeriodSequence.GetHashCode();
                if (this.SchoolReference != null)
                    hashCode = hashCode * 59 + this.SchoolReference.GetHashCode();
                if (this.SchoolYearTypeReference != null)
                    hashCode = hashCode * 59 + this.SchoolYearTypeReference.GetHashCode();
                if (this.BeginDate != null)
                    hashCode = hashCode * 59 + this.BeginDate.GetHashCode();
                if (this.EndDate != null)
                    hashCode = hashCode * 59 + this.EndDate.GetHashCode();
                if (this.TotalInstructionalDays != null)
                    hashCode = hashCode * 59 + this.TotalInstructionalDays.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // GradingPeriodDescriptor (string) maxLength
            if(this.GradingPeriodDescriptor != null && this.GradingPeriodDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for GradingPeriodDescriptor, length must be less than 306.", new [] { "GradingPeriodDescriptor" });
            }

            yield break;
        }
    }

}
