/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiInterventionStudy
    /// </summary>
    [DataContract]
    public partial class EdFiInterventionStudy :  IEquatable<EdFiInterventionStudy>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiInterventionStudy" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiInterventionStudy() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiInterventionStudy" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="InterventionStudyIdentificationCode">A unique number or alphanumeric code assigned to an intervention study. (required).</param>
        /// <param name="EducationOrganizationReference">EducationOrganizationReference (required).</param>
        /// <param name="InterventionPrescriptionReference">InterventionPrescriptionReference (required).</param>
        /// <param name="AppropriateGradeLevels">An unordered collection of interventionStudyAppropriateGradeLevels. Grade levels participating in this study..</param>
        /// <param name="AppropriateSexes">An unordered collection of interventionStudyAppropriateSexes. Sexes participating in this study. If omitted, considered generally applicable..</param>
        /// <param name="DeliveryMethodDescriptor">The way in which an intervention was implemented: individual, small group, whole class, or whole school. (required).</param>
        /// <param name="EducationContents">An unordered collection of interventionStudyEducationContents. Relates the education content source to the education content..</param>
        /// <param name="InterventionClassDescriptor">The way in which an intervention is used: curriculum, supplement, or practice. (required).</param>
        /// <param name="InterventionEffectivenesses">An unordered collection of interventionStudyInterventionEffectivenesses. Measurement of the effectiveness of the InterventionStudy per diagnosis..</param>
        /// <param name="LearningResourceMetadataURIs">An unordered collection of interventionStudyLearningResourceMetadataURIs. The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item..</param>
        /// <param name="Participants">The number of participants observed in the study. (required).</param>
        /// <param name="PopulationServeds">An unordered collection of interventionStudyPopulationServeds. A subset of students that are the focus of the InterventionStudy..</param>
        /// <param name="StateAbbreviations">An unordered collection of interventionStudyStateAbbreviations. The abbreviation for the state (within the United States) or outlying area, the school system of which the participants of the study are considered to be a part..</param>
        /// <param name="Uris">An unordered collection of interventionStudyURIs. The URI (typical a URL) pointing to an education content item..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiInterventionStudy(string Id = default(string), string InterventionStudyIdentificationCode = default(string), EdFiEducationOrganizationReference EducationOrganizationReference = default(EdFiEducationOrganizationReference), EdFiInterventionPrescriptionReference InterventionPrescriptionReference = default(EdFiInterventionPrescriptionReference), List<EdFiInterventionStudyAppropriateGradeLevel> AppropriateGradeLevels = default(List<EdFiInterventionStudyAppropriateGradeLevel>), List<EdFiInterventionStudyAppropriateSex> AppropriateSexes = default(List<EdFiInterventionStudyAppropriateSex>), string DeliveryMethodDescriptor = default(string), List<EdFiInterventionStudyEducationContent> EducationContents = default(List<EdFiInterventionStudyEducationContent>), string InterventionClassDescriptor = default(string), List<EdFiInterventionStudyInterventionEffectiveness> InterventionEffectivenesses = default(List<EdFiInterventionStudyInterventionEffectiveness>), List<EdFiInterventionStudyLearningResourceMetadataURI> LearningResourceMetadataURIs = default(List<EdFiInterventionStudyLearningResourceMetadataURI>), int? Participants = default(int?), List<EdFiInterventionStudyPopulationServed> PopulationServeds = default(List<EdFiInterventionStudyPopulationServed>), List<EdFiInterventionStudyStateAbbreviation> StateAbbreviations = default(List<EdFiInterventionStudyStateAbbreviation>), List<EdFiInterventionStudyURI> Uris = default(List<EdFiInterventionStudyURI>), string Etag = default(string))
        {
            // to ensure "InterventionStudyIdentificationCode" is required (not null)
            if (InterventionStudyIdentificationCode == null)
            {
                throw new InvalidDataException("InterventionStudyIdentificationCode is a required property for EdFiInterventionStudy and cannot be null");
            }
            else
            {
                this.InterventionStudyIdentificationCode = InterventionStudyIdentificationCode;
            }
            // to ensure "EducationOrganizationReference" is required (not null)
            if (EducationOrganizationReference == null)
            {
                throw new InvalidDataException("EducationOrganizationReference is a required property for EdFiInterventionStudy and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = EducationOrganizationReference;
            }
            // to ensure "InterventionPrescriptionReference" is required (not null)
            if (InterventionPrescriptionReference == null)
            {
                throw new InvalidDataException("InterventionPrescriptionReference is a required property for EdFiInterventionStudy and cannot be null");
            }
            else
            {
                this.InterventionPrescriptionReference = InterventionPrescriptionReference;
            }
            // to ensure "DeliveryMethodDescriptor" is required (not null)
            if (DeliveryMethodDescriptor == null)
            {
                throw new InvalidDataException("DeliveryMethodDescriptor is a required property for EdFiInterventionStudy and cannot be null");
            }
            else
            {
                this.DeliveryMethodDescriptor = DeliveryMethodDescriptor;
            }
            // to ensure "InterventionClassDescriptor" is required (not null)
            if (InterventionClassDescriptor == null)
            {
                throw new InvalidDataException("InterventionClassDescriptor is a required property for EdFiInterventionStudy and cannot be null");
            }
            else
            {
                this.InterventionClassDescriptor = InterventionClassDescriptor;
            }
            // to ensure "Participants" is required (not null)
            if (Participants == null)
            {
                throw new InvalidDataException("Participants is a required property for EdFiInterventionStudy and cannot be null");
            }
            else
            {
                this.Participants = Participants;
            }
            this.Id = Id;
            this.AppropriateGradeLevels = AppropriateGradeLevels;
            this.AppropriateSexes = AppropriateSexes;
            this.EducationContents = EducationContents;
            this.InterventionEffectivenesses = InterventionEffectivenesses;
            this.LearningResourceMetadataURIs = LearningResourceMetadataURIs;
            this.PopulationServeds = PopulationServeds;
            this.StateAbbreviations = StateAbbreviations;
            this.Uris = Uris;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to an intervention study.
        /// </summary>
        /// <value>A unique number or alphanumeric code assigned to an intervention study.</value>
        [DataMember(Name="interventionStudyIdentificationCode", EmitDefaultValue=false)]
        public string InterventionStudyIdentificationCode { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// Gets or Sets InterventionPrescriptionReference
        /// </summary>
        [DataMember(Name="interventionPrescriptionReference", EmitDefaultValue=false)]
        public EdFiInterventionPrescriptionReference InterventionPrescriptionReference { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyAppropriateGradeLevels. Grade levels participating in this study.
        /// </summary>
        /// <value>An unordered collection of interventionStudyAppropriateGradeLevels. Grade levels participating in this study.</value>
        [DataMember(Name="appropriateGradeLevels", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyAppropriateGradeLevel> AppropriateGradeLevels { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyAppropriateSexes. Sexes participating in this study. If omitted, considered generally applicable.
        /// </summary>
        /// <value>An unordered collection of interventionStudyAppropriateSexes. Sexes participating in this study. If omitted, considered generally applicable.</value>
        [DataMember(Name="appropriateSexes", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyAppropriateSex> AppropriateSexes { get; set; }

        /// <summary>
        /// The way in which an intervention was implemented: individual, small group, whole class, or whole school.
        /// </summary>
        /// <value>The way in which an intervention was implemented: individual, small group, whole class, or whole school.</value>
        [DataMember(Name="deliveryMethodDescriptor", EmitDefaultValue=false)]
        public string DeliveryMethodDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyEducationContents. Relates the education content source to the education content.
        /// </summary>
        /// <value>An unordered collection of interventionStudyEducationContents. Relates the education content source to the education content.</value>
        [DataMember(Name="educationContents", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyEducationContent> EducationContents { get; set; }

        /// <summary>
        /// The way in which an intervention is used: curriculum, supplement, or practice.
        /// </summary>
        /// <value>The way in which an intervention is used: curriculum, supplement, or practice.</value>
        [DataMember(Name="interventionClassDescriptor", EmitDefaultValue=false)]
        public string InterventionClassDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyInterventionEffectivenesses. Measurement of the effectiveness of the InterventionStudy per diagnosis.
        /// </summary>
        /// <value>An unordered collection of interventionStudyInterventionEffectivenesses. Measurement of the effectiveness of the InterventionStudy per diagnosis.</value>
        [DataMember(Name="interventionEffectivenesses", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyInterventionEffectiveness> InterventionEffectivenesses { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyLearningResourceMetadataURIs. The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item.
        /// </summary>
        /// <value>An unordered collection of interventionStudyLearningResourceMetadataURIs. The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item.</value>
        [DataMember(Name="learningResourceMetadataURIs", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyLearningResourceMetadataURI> LearningResourceMetadataURIs { get; set; }

        /// <summary>
        /// The number of participants observed in the study.
        /// </summary>
        /// <value>The number of participants observed in the study.</value>
        [DataMember(Name="participants", EmitDefaultValue=false)]
        public int? Participants { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyPopulationServeds. A subset of students that are the focus of the InterventionStudy.
        /// </summary>
        /// <value>An unordered collection of interventionStudyPopulationServeds. A subset of students that are the focus of the InterventionStudy.</value>
        [DataMember(Name="populationServeds", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyPopulationServed> PopulationServeds { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyStateAbbreviations. The abbreviation for the state (within the United States) or outlying area, the school system of which the participants of the study are considered to be a part.
        /// </summary>
        /// <value>An unordered collection of interventionStudyStateAbbreviations. The abbreviation for the state (within the United States) or outlying area, the school system of which the participants of the study are considered to be a part.</value>
        [DataMember(Name="stateAbbreviations", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyStateAbbreviation> StateAbbreviations { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyURIs. The URI (typical a URL) pointing to an education content item.
        /// </summary>
        /// <value>An unordered collection of interventionStudyURIs. The URI (typical a URL) pointing to an education content item.</value>
        [DataMember(Name="uris", EmitDefaultValue=false)]
        public List<EdFiInterventionStudyURI> Uris { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiInterventionStudy {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  InterventionStudyIdentificationCode: ").Append(InterventionStudyIdentificationCode).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  InterventionPrescriptionReference: ").Append(InterventionPrescriptionReference).Append("\n");
            sb.Append("  AppropriateGradeLevels: ").Append(AppropriateGradeLevels).Append("\n");
            sb.Append("  AppropriateSexes: ").Append(AppropriateSexes).Append("\n");
            sb.Append("  DeliveryMethodDescriptor: ").Append(DeliveryMethodDescriptor).Append("\n");
            sb.Append("  EducationContents: ").Append(EducationContents).Append("\n");
            sb.Append("  InterventionClassDescriptor: ").Append(InterventionClassDescriptor).Append("\n");
            sb.Append("  InterventionEffectivenesses: ").Append(InterventionEffectivenesses).Append("\n");
            sb.Append("  LearningResourceMetadataURIs: ").Append(LearningResourceMetadataURIs).Append("\n");
            sb.Append("  Participants: ").Append(Participants).Append("\n");
            sb.Append("  PopulationServeds: ").Append(PopulationServeds).Append("\n");
            sb.Append("  StateAbbreviations: ").Append(StateAbbreviations).Append("\n");
            sb.Append("  Uris: ").Append(Uris).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiInterventionStudy);
        }

        /// <summary>
        /// Returns true if EdFiInterventionStudy instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiInterventionStudy to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiInterventionStudy input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.InterventionStudyIdentificationCode == input.InterventionStudyIdentificationCode ||
                    (this.InterventionStudyIdentificationCode != null &&
                    this.InterventionStudyIdentificationCode.Equals(input.InterventionStudyIdentificationCode))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.InterventionPrescriptionReference == input.InterventionPrescriptionReference ||
                    (this.InterventionPrescriptionReference != null &&
                    this.InterventionPrescriptionReference.Equals(input.InterventionPrescriptionReference))
                ) && 
                (
                    this.AppropriateGradeLevels == input.AppropriateGradeLevels ||
                    this.AppropriateGradeLevels != null &&
                    this.AppropriateGradeLevels.SequenceEqual(input.AppropriateGradeLevels)
                ) && 
                (
                    this.AppropriateSexes == input.AppropriateSexes ||
                    this.AppropriateSexes != null &&
                    this.AppropriateSexes.SequenceEqual(input.AppropriateSexes)
                ) && 
                (
                    this.DeliveryMethodDescriptor == input.DeliveryMethodDescriptor ||
                    (this.DeliveryMethodDescriptor != null &&
                    this.DeliveryMethodDescriptor.Equals(input.DeliveryMethodDescriptor))
                ) && 
                (
                    this.EducationContents == input.EducationContents ||
                    this.EducationContents != null &&
                    this.EducationContents.SequenceEqual(input.EducationContents)
                ) && 
                (
                    this.InterventionClassDescriptor == input.InterventionClassDescriptor ||
                    (this.InterventionClassDescriptor != null &&
                    this.InterventionClassDescriptor.Equals(input.InterventionClassDescriptor))
                ) && 
                (
                    this.InterventionEffectivenesses == input.InterventionEffectivenesses ||
                    this.InterventionEffectivenesses != null &&
                    this.InterventionEffectivenesses.SequenceEqual(input.InterventionEffectivenesses)
                ) && 
                (
                    this.LearningResourceMetadataURIs == input.LearningResourceMetadataURIs ||
                    this.LearningResourceMetadataURIs != null &&
                    this.LearningResourceMetadataURIs.SequenceEqual(input.LearningResourceMetadataURIs)
                ) && 
                (
                    this.Participants == input.Participants ||
                    (this.Participants != null &&
                    this.Participants.Equals(input.Participants))
                ) && 
                (
                    this.PopulationServeds == input.PopulationServeds ||
                    this.PopulationServeds != null &&
                    this.PopulationServeds.SequenceEqual(input.PopulationServeds)
                ) && 
                (
                    this.StateAbbreviations == input.StateAbbreviations ||
                    this.StateAbbreviations != null &&
                    this.StateAbbreviations.SequenceEqual(input.StateAbbreviations)
                ) && 
                (
                    this.Uris == input.Uris ||
                    this.Uris != null &&
                    this.Uris.SequenceEqual(input.Uris)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.InterventionStudyIdentificationCode != null)
                    hashCode = hashCode * 59 + this.InterventionStudyIdentificationCode.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.InterventionPrescriptionReference != null)
                    hashCode = hashCode * 59 + this.InterventionPrescriptionReference.GetHashCode();
                if (this.AppropriateGradeLevels != null)
                    hashCode = hashCode * 59 + this.AppropriateGradeLevels.GetHashCode();
                if (this.AppropriateSexes != null)
                    hashCode = hashCode * 59 + this.AppropriateSexes.GetHashCode();
                if (this.DeliveryMethodDescriptor != null)
                    hashCode = hashCode * 59 + this.DeliveryMethodDescriptor.GetHashCode();
                if (this.EducationContents != null)
                    hashCode = hashCode * 59 + this.EducationContents.GetHashCode();
                if (this.InterventionClassDescriptor != null)
                    hashCode = hashCode * 59 + this.InterventionClassDescriptor.GetHashCode();
                if (this.InterventionEffectivenesses != null)
                    hashCode = hashCode * 59 + this.InterventionEffectivenesses.GetHashCode();
                if (this.LearningResourceMetadataURIs != null)
                    hashCode = hashCode * 59 + this.LearningResourceMetadataURIs.GetHashCode();
                if (this.Participants != null)
                    hashCode = hashCode * 59 + this.Participants.GetHashCode();
                if (this.PopulationServeds != null)
                    hashCode = hashCode * 59 + this.PopulationServeds.GetHashCode();
                if (this.StateAbbreviations != null)
                    hashCode = hashCode * 59 + this.StateAbbreviations.GetHashCode();
                if (this.Uris != null)
                    hashCode = hashCode * 59 + this.Uris.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // InterventionStudyIdentificationCode (string) maxLength
            if(this.InterventionStudyIdentificationCode != null && this.InterventionStudyIdentificationCode.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for InterventionStudyIdentificationCode, length must be less than 60.", new [] { "InterventionStudyIdentificationCode" });
            }

            // DeliveryMethodDescriptor (string) maxLength
            if(this.DeliveryMethodDescriptor != null && this.DeliveryMethodDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DeliveryMethodDescriptor, length must be less than 306.", new [] { "DeliveryMethodDescriptor" });
            }

            // InterventionClassDescriptor (string) maxLength
            if(this.InterventionClassDescriptor != null && this.InterventionClassDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for InterventionClassDescriptor, length must be less than 306.", new [] { "InterventionClassDescriptor" });
            }

            yield break;
        }
    }

}
