/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiSection
    /// </summary>
    [DataContract]
    public partial class EdFiSection :  IEquatable<EdFiSection>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSection" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiSection() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSection" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="SectionIdentifier">The local identifier assigned to a section. (required).</param>
        /// <param name="CourseOfferingReference">CourseOfferingReference (required).</param>
        /// <param name="LocationReference">LocationReference.</param>
        /// <param name="LocationSchoolReference">LocationSchoolReference.</param>
        /// <param name="AvailableCreditConversion">Conversion factor that when multiplied by the number of credits is equivalent to Carnegie units..</param>
        /// <param name="AvailableCredits">The value of credits or units of value awarded for the completion of a course..</param>
        /// <param name="AvailableCreditTypeDescriptor">The type of credits or units of value awarded for the completion of a course..</param>
        /// <param name="Characteristics">An unordered collection of sectionCharacteristics. Reflects important characteristics of the Section, such as whether or not attendance is taken and the Section is graded..</param>
        /// <param name="ClassPeriods">An unordered collection of sectionClassPeriods. The class period during which the Section meets..</param>
        /// <param name="CourseLevelCharacteristics">An unordered collection of sectionCourseLevelCharacteristics. The type of specific program or designation with which the section is associated (e.g., AP, IB, Dual Credit, CTE). This collection should only be populated if it differs from the Course Level Characteristics identified at the Course Offering level..</param>
        /// <param name="EducationalEnvironmentDescriptor">The setting in which a child receives education and related services; for example:          Center-based instruction          Home-based instruction          Hospital class          Mainstream          Residential care and treatment facility          ....</param>
        /// <param name="InstructionLanguageDescriptor">The primary language of instruction, if omitted English is assumed..</param>
        /// <param name="MediumOfInstructionDescriptor">The media through which teachers provide instruction to students and students and teachers communicate about instructional matters; for example:          Technology-based instruction in classroom          Correspondence instruction          Face-to-face instruction          Virtual/On-line Distance learning          Center-based instruction          ....</param>
        /// <param name="OfferedGradeLevels">An unordered collection of sectionOfferedGradeLevels. The grade levels in which the section is offered. This collection should only be populated if it differs from the Offered Grade Levels identified at the Course Offering level..</param>
        /// <param name="OfficialAttendancePeriod">Indicator of whether this section is used for official daily attendance. Alternatively, official daily attendance may be tied to a Class Period..</param>
        /// <param name="PopulationServedDescriptor">The type of students the Section is offered and tailored to; for example:          Bilingual students          Remedial education students          Gifted and talented students          Career and Technical Education students          Special education students          ....</param>
        /// <param name="Programs">An unordered collection of sectionPrograms. Optional reference to program (e.g., CTE) to which the Section is associated..</param>
        /// <param name="SectionName">A locally-defined name for the section, generally created to make the section more recognizable in informal contexts and generally distinct from the SectionIdentifier..</param>
        /// <param name="SequenceOfCourse">When a section is part of a sequence of parts for a course, the number of the sequence. If the course has only one part, the value of this section attribute should be 1..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiSection(string Id = default(string), string SectionIdentifier = default(string), EdFiCourseOfferingReference CourseOfferingReference = default(EdFiCourseOfferingReference), EdFiLocationReference LocationReference = default(EdFiLocationReference), EdFiSchoolReference LocationSchoolReference = default(EdFiSchoolReference), double? AvailableCreditConversion = default(double?), double? AvailableCredits = default(double?), string AvailableCreditTypeDescriptor = default(string), List<EdFiSectionCharacteristic> Characteristics = default(List<EdFiSectionCharacteristic>), List<EdFiSectionClassPeriod> ClassPeriods = default(List<EdFiSectionClassPeriod>), List<EdFiSectionCourseLevelCharacteristic> CourseLevelCharacteristics = default(List<EdFiSectionCourseLevelCharacteristic>), string EducationalEnvironmentDescriptor = default(string), string InstructionLanguageDescriptor = default(string), string MediumOfInstructionDescriptor = default(string), List<EdFiSectionOfferedGradeLevel> OfferedGradeLevels = default(List<EdFiSectionOfferedGradeLevel>), bool? OfficialAttendancePeriod = default(bool?), string PopulationServedDescriptor = default(string), List<EdFiSectionProgram> Programs = default(List<EdFiSectionProgram>), string SectionName = default(string), int? SequenceOfCourse = default(int?), string Etag = default(string))
        {
            // to ensure "SectionIdentifier" is required (not null)
            if (SectionIdentifier == null)
            {
                throw new InvalidDataException("SectionIdentifier is a required property for EdFiSection and cannot be null");
            }
            else
            {
                this.SectionIdentifier = SectionIdentifier;
            }
            // to ensure "CourseOfferingReference" is required (not null)
            if (CourseOfferingReference == null)
            {
                throw new InvalidDataException("CourseOfferingReference is a required property for EdFiSection and cannot be null");
            }
            else
            {
                this.CourseOfferingReference = CourseOfferingReference;
            }
            this.Id = Id;
            this.LocationReference = LocationReference;
            this.LocationSchoolReference = LocationSchoolReference;
            this.AvailableCreditConversion = AvailableCreditConversion;
            this.AvailableCredits = AvailableCredits;
            this.AvailableCreditTypeDescriptor = AvailableCreditTypeDescriptor;
            this.Characteristics = Characteristics;
            this.ClassPeriods = ClassPeriods;
            this.CourseLevelCharacteristics = CourseLevelCharacteristics;
            this.EducationalEnvironmentDescriptor = EducationalEnvironmentDescriptor;
            this.InstructionLanguageDescriptor = InstructionLanguageDescriptor;
            this.MediumOfInstructionDescriptor = MediumOfInstructionDescriptor;
            this.OfferedGradeLevels = OfferedGradeLevels;
            this.OfficialAttendancePeriod = OfficialAttendancePeriod;
            this.PopulationServedDescriptor = PopulationServedDescriptor;
            this.Programs = Programs;
            this.SectionName = SectionName;
            this.SequenceOfCourse = SequenceOfCourse;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The local identifier assigned to a section.
        /// </summary>
        /// <value>The local identifier assigned to a section.</value>
        [DataMember(Name="sectionIdentifier", EmitDefaultValue=false)]
        public string SectionIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets CourseOfferingReference
        /// </summary>
        [DataMember(Name="courseOfferingReference", EmitDefaultValue=false)]
        public EdFiCourseOfferingReference CourseOfferingReference { get; set; }

        /// <summary>
        /// Gets or Sets LocationReference
        /// </summary>
        [DataMember(Name="locationReference", EmitDefaultValue=false)]
        public EdFiLocationReference LocationReference { get; set; }

        /// <summary>
        /// Gets or Sets LocationSchoolReference
        /// </summary>
        [DataMember(Name="locationSchoolReference", EmitDefaultValue=false)]
        public EdFiSchoolReference LocationSchoolReference { get; set; }

        /// <summary>
        /// Conversion factor that when multiplied by the number of credits is equivalent to Carnegie units.
        /// </summary>
        /// <value>Conversion factor that when multiplied by the number of credits is equivalent to Carnegie units.</value>
        [DataMember(Name="availableCreditConversion", EmitDefaultValue=false)]
        public double? AvailableCreditConversion { get; set; }

        /// <summary>
        /// The value of credits or units of value awarded for the completion of a course.
        /// </summary>
        /// <value>The value of credits or units of value awarded for the completion of a course.</value>
        [DataMember(Name="availableCredits", EmitDefaultValue=false)]
        public double? AvailableCredits { get; set; }

        /// <summary>
        /// The type of credits or units of value awarded for the completion of a course.
        /// </summary>
        /// <value>The type of credits or units of value awarded for the completion of a course.</value>
        [DataMember(Name="availableCreditTypeDescriptor", EmitDefaultValue=false)]
        public string AvailableCreditTypeDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of sectionCharacteristics. Reflects important characteristics of the Section, such as whether or not attendance is taken and the Section is graded.
        /// </summary>
        /// <value>An unordered collection of sectionCharacteristics. Reflects important characteristics of the Section, such as whether or not attendance is taken and the Section is graded.</value>
        [DataMember(Name="characteristics", EmitDefaultValue=false)]
        public List<EdFiSectionCharacteristic> Characteristics { get; set; }

        /// <summary>
        /// An unordered collection of sectionClassPeriods. The class period during which the Section meets.
        /// </summary>
        /// <value>An unordered collection of sectionClassPeriods. The class period during which the Section meets.</value>
        [DataMember(Name="classPeriods", EmitDefaultValue=false)]
        public List<EdFiSectionClassPeriod> ClassPeriods { get; set; }

        /// <summary>
        /// An unordered collection of sectionCourseLevelCharacteristics. The type of specific program or designation with which the section is associated (e.g., AP, IB, Dual Credit, CTE). This collection should only be populated if it differs from the Course Level Characteristics identified at the Course Offering level.
        /// </summary>
        /// <value>An unordered collection of sectionCourseLevelCharacteristics. The type of specific program or designation with which the section is associated (e.g., AP, IB, Dual Credit, CTE). This collection should only be populated if it differs from the Course Level Characteristics identified at the Course Offering level.</value>
        [DataMember(Name="courseLevelCharacteristics", EmitDefaultValue=false)]
        public List<EdFiSectionCourseLevelCharacteristic> CourseLevelCharacteristics { get; set; }

        /// <summary>
        /// The setting in which a child receives education and related services; for example:          Center-based instruction          Home-based instruction          Hospital class          Mainstream          Residential care and treatment facility          ...
        /// </summary>
        /// <value>The setting in which a child receives education and related services; for example:          Center-based instruction          Home-based instruction          Hospital class          Mainstream          Residential care and treatment facility          ...</value>
        [DataMember(Name="educationalEnvironmentDescriptor", EmitDefaultValue=false)]
        public string EducationalEnvironmentDescriptor { get; set; }

        /// <summary>
        /// The primary language of instruction, if omitted English is assumed.
        /// </summary>
        /// <value>The primary language of instruction, if omitted English is assumed.</value>
        [DataMember(Name="instructionLanguageDescriptor", EmitDefaultValue=false)]
        public string InstructionLanguageDescriptor { get; set; }

        /// <summary>
        /// The media through which teachers provide instruction to students and students and teachers communicate about instructional matters; for example:          Technology-based instruction in classroom          Correspondence instruction          Face-to-face instruction          Virtual/On-line Distance learning          Center-based instruction          ...
        /// </summary>
        /// <value>The media through which teachers provide instruction to students and students and teachers communicate about instructional matters; for example:          Technology-based instruction in classroom          Correspondence instruction          Face-to-face instruction          Virtual/On-line Distance learning          Center-based instruction          ...</value>
        [DataMember(Name="mediumOfInstructionDescriptor", EmitDefaultValue=false)]
        public string MediumOfInstructionDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of sectionOfferedGradeLevels. The grade levels in which the section is offered. This collection should only be populated if it differs from the Offered Grade Levels identified at the Course Offering level.
        /// </summary>
        /// <value>An unordered collection of sectionOfferedGradeLevels. The grade levels in which the section is offered. This collection should only be populated if it differs from the Offered Grade Levels identified at the Course Offering level.</value>
        [DataMember(Name="offeredGradeLevels", EmitDefaultValue=false)]
        public List<EdFiSectionOfferedGradeLevel> OfferedGradeLevels { get; set; }

        /// <summary>
        /// Indicator of whether this section is used for official daily attendance. Alternatively, official daily attendance may be tied to a Class Period.
        /// </summary>
        /// <value>Indicator of whether this section is used for official daily attendance. Alternatively, official daily attendance may be tied to a Class Period.</value>
        [DataMember(Name="officialAttendancePeriod", EmitDefaultValue=false)]
        public bool? OfficialAttendancePeriod { get; set; }

        /// <summary>
        /// The type of students the Section is offered and tailored to; for example:          Bilingual students          Remedial education students          Gifted and talented students          Career and Technical Education students          Special education students          ...
        /// </summary>
        /// <value>The type of students the Section is offered and tailored to; for example:          Bilingual students          Remedial education students          Gifted and talented students          Career and Technical Education students          Special education students          ...</value>
        [DataMember(Name="populationServedDescriptor", EmitDefaultValue=false)]
        public string PopulationServedDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of sectionPrograms. Optional reference to program (e.g., CTE) to which the Section is associated.
        /// </summary>
        /// <value>An unordered collection of sectionPrograms. Optional reference to program (e.g., CTE) to which the Section is associated.</value>
        [DataMember(Name="programs", EmitDefaultValue=false)]
        public List<EdFiSectionProgram> Programs { get; set; }

        /// <summary>
        /// A locally-defined name for the section, generally created to make the section more recognizable in informal contexts and generally distinct from the SectionIdentifier.
        /// </summary>
        /// <value>A locally-defined name for the section, generally created to make the section more recognizable in informal contexts and generally distinct from the SectionIdentifier.</value>
        [DataMember(Name="sectionName", EmitDefaultValue=false)]
        public string SectionName { get; set; }

        /// <summary>
        /// When a section is part of a sequence of parts for a course, the number of the sequence. If the course has only one part, the value of this section attribute should be 1.
        /// </summary>
        /// <value>When a section is part of a sequence of parts for a course, the number of the sequence. If the course has only one part, the value of this section attribute should be 1.</value>
        [DataMember(Name="sequenceOfCourse", EmitDefaultValue=false)]
        public int? SequenceOfCourse { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiSection {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  SectionIdentifier: ").Append(SectionIdentifier).Append("\n");
            sb.Append("  CourseOfferingReference: ").Append(CourseOfferingReference).Append("\n");
            sb.Append("  LocationReference: ").Append(LocationReference).Append("\n");
            sb.Append("  LocationSchoolReference: ").Append(LocationSchoolReference).Append("\n");
            sb.Append("  AvailableCreditConversion: ").Append(AvailableCreditConversion).Append("\n");
            sb.Append("  AvailableCredits: ").Append(AvailableCredits).Append("\n");
            sb.Append("  AvailableCreditTypeDescriptor: ").Append(AvailableCreditTypeDescriptor).Append("\n");
            sb.Append("  Characteristics: ").Append(Characteristics).Append("\n");
            sb.Append("  ClassPeriods: ").Append(ClassPeriods).Append("\n");
            sb.Append("  CourseLevelCharacteristics: ").Append(CourseLevelCharacteristics).Append("\n");
            sb.Append("  EducationalEnvironmentDescriptor: ").Append(EducationalEnvironmentDescriptor).Append("\n");
            sb.Append("  InstructionLanguageDescriptor: ").Append(InstructionLanguageDescriptor).Append("\n");
            sb.Append("  MediumOfInstructionDescriptor: ").Append(MediumOfInstructionDescriptor).Append("\n");
            sb.Append("  OfferedGradeLevels: ").Append(OfferedGradeLevels).Append("\n");
            sb.Append("  OfficialAttendancePeriod: ").Append(OfficialAttendancePeriod).Append("\n");
            sb.Append("  PopulationServedDescriptor: ").Append(PopulationServedDescriptor).Append("\n");
            sb.Append("  Programs: ").Append(Programs).Append("\n");
            sb.Append("  SectionName: ").Append(SectionName).Append("\n");
            sb.Append("  SequenceOfCourse: ").Append(SequenceOfCourse).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiSection);
        }

        /// <summary>
        /// Returns true if EdFiSection instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiSection to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiSection input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.SectionIdentifier == input.SectionIdentifier ||
                    (this.SectionIdentifier != null &&
                    this.SectionIdentifier.Equals(input.SectionIdentifier))
                ) && 
                (
                    this.CourseOfferingReference == input.CourseOfferingReference ||
                    (this.CourseOfferingReference != null &&
                    this.CourseOfferingReference.Equals(input.CourseOfferingReference))
                ) && 
                (
                    this.LocationReference == input.LocationReference ||
                    (this.LocationReference != null &&
                    this.LocationReference.Equals(input.LocationReference))
                ) && 
                (
                    this.LocationSchoolReference == input.LocationSchoolReference ||
                    (this.LocationSchoolReference != null &&
                    this.LocationSchoolReference.Equals(input.LocationSchoolReference))
                ) && 
                (
                    this.AvailableCreditConversion == input.AvailableCreditConversion ||
                    (this.AvailableCreditConversion != null &&
                    this.AvailableCreditConversion.Equals(input.AvailableCreditConversion))
                ) && 
                (
                    this.AvailableCredits == input.AvailableCredits ||
                    (this.AvailableCredits != null &&
                    this.AvailableCredits.Equals(input.AvailableCredits))
                ) && 
                (
                    this.AvailableCreditTypeDescriptor == input.AvailableCreditTypeDescriptor ||
                    (this.AvailableCreditTypeDescriptor != null &&
                    this.AvailableCreditTypeDescriptor.Equals(input.AvailableCreditTypeDescriptor))
                ) && 
                (
                    this.Characteristics == input.Characteristics ||
                    this.Characteristics != null &&
                    this.Characteristics.SequenceEqual(input.Characteristics)
                ) && 
                (
                    this.ClassPeriods == input.ClassPeriods ||
                    this.ClassPeriods != null &&
                    this.ClassPeriods.SequenceEqual(input.ClassPeriods)
                ) && 
                (
                    this.CourseLevelCharacteristics == input.CourseLevelCharacteristics ||
                    this.CourseLevelCharacteristics != null &&
                    this.CourseLevelCharacteristics.SequenceEqual(input.CourseLevelCharacteristics)
                ) && 
                (
                    this.EducationalEnvironmentDescriptor == input.EducationalEnvironmentDescriptor ||
                    (this.EducationalEnvironmentDescriptor != null &&
                    this.EducationalEnvironmentDescriptor.Equals(input.EducationalEnvironmentDescriptor))
                ) && 
                (
                    this.InstructionLanguageDescriptor == input.InstructionLanguageDescriptor ||
                    (this.InstructionLanguageDescriptor != null &&
                    this.InstructionLanguageDescriptor.Equals(input.InstructionLanguageDescriptor))
                ) && 
                (
                    this.MediumOfInstructionDescriptor == input.MediumOfInstructionDescriptor ||
                    (this.MediumOfInstructionDescriptor != null &&
                    this.MediumOfInstructionDescriptor.Equals(input.MediumOfInstructionDescriptor))
                ) && 
                (
                    this.OfferedGradeLevels == input.OfferedGradeLevels ||
                    this.OfferedGradeLevels != null &&
                    this.OfferedGradeLevels.SequenceEqual(input.OfferedGradeLevels)
                ) && 
                (
                    this.OfficialAttendancePeriod == input.OfficialAttendancePeriod ||
                    (this.OfficialAttendancePeriod != null &&
                    this.OfficialAttendancePeriod.Equals(input.OfficialAttendancePeriod))
                ) && 
                (
                    this.PopulationServedDescriptor == input.PopulationServedDescriptor ||
                    (this.PopulationServedDescriptor != null &&
                    this.PopulationServedDescriptor.Equals(input.PopulationServedDescriptor))
                ) && 
                (
                    this.Programs == input.Programs ||
                    this.Programs != null &&
                    this.Programs.SequenceEqual(input.Programs)
                ) && 
                (
                    this.SectionName == input.SectionName ||
                    (this.SectionName != null &&
                    this.SectionName.Equals(input.SectionName))
                ) && 
                (
                    this.SequenceOfCourse == input.SequenceOfCourse ||
                    (this.SequenceOfCourse != null &&
                    this.SequenceOfCourse.Equals(input.SequenceOfCourse))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.SectionIdentifier != null)
                    hashCode = hashCode * 59 + this.SectionIdentifier.GetHashCode();
                if (this.CourseOfferingReference != null)
                    hashCode = hashCode * 59 + this.CourseOfferingReference.GetHashCode();
                if (this.LocationReference != null)
                    hashCode = hashCode * 59 + this.LocationReference.GetHashCode();
                if (this.LocationSchoolReference != null)
                    hashCode = hashCode * 59 + this.LocationSchoolReference.GetHashCode();
                if (this.AvailableCreditConversion != null)
                    hashCode = hashCode * 59 + this.AvailableCreditConversion.GetHashCode();
                if (this.AvailableCredits != null)
                    hashCode = hashCode * 59 + this.AvailableCredits.GetHashCode();
                if (this.AvailableCreditTypeDescriptor != null)
                    hashCode = hashCode * 59 + this.AvailableCreditTypeDescriptor.GetHashCode();
                if (this.Characteristics != null)
                    hashCode = hashCode * 59 + this.Characteristics.GetHashCode();
                if (this.ClassPeriods != null)
                    hashCode = hashCode * 59 + this.ClassPeriods.GetHashCode();
                if (this.CourseLevelCharacteristics != null)
                    hashCode = hashCode * 59 + this.CourseLevelCharacteristics.GetHashCode();
                if (this.EducationalEnvironmentDescriptor != null)
                    hashCode = hashCode * 59 + this.EducationalEnvironmentDescriptor.GetHashCode();
                if (this.InstructionLanguageDescriptor != null)
                    hashCode = hashCode * 59 + this.InstructionLanguageDescriptor.GetHashCode();
                if (this.MediumOfInstructionDescriptor != null)
                    hashCode = hashCode * 59 + this.MediumOfInstructionDescriptor.GetHashCode();
                if (this.OfferedGradeLevels != null)
                    hashCode = hashCode * 59 + this.OfferedGradeLevels.GetHashCode();
                if (this.OfficialAttendancePeriod != null)
                    hashCode = hashCode * 59 + this.OfficialAttendancePeriod.GetHashCode();
                if (this.PopulationServedDescriptor != null)
                    hashCode = hashCode * 59 + this.PopulationServedDescriptor.GetHashCode();
                if (this.Programs != null)
                    hashCode = hashCode * 59 + this.Programs.GetHashCode();
                if (this.SectionName != null)
                    hashCode = hashCode * 59 + this.SectionName.GetHashCode();
                if (this.SequenceOfCourse != null)
                    hashCode = hashCode * 59 + this.SequenceOfCourse.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // SectionIdentifier (string) maxLength
            if(this.SectionIdentifier != null && this.SectionIdentifier.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SectionIdentifier, length must be less than 255.", new [] { "SectionIdentifier" });
            }

            // AvailableCreditTypeDescriptor (string) maxLength
            if(this.AvailableCreditTypeDescriptor != null && this.AvailableCreditTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AvailableCreditTypeDescriptor, length must be less than 306.", new [] { "AvailableCreditTypeDescriptor" });
            }

            // EducationalEnvironmentDescriptor (string) maxLength
            if(this.EducationalEnvironmentDescriptor != null && this.EducationalEnvironmentDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for EducationalEnvironmentDescriptor, length must be less than 306.", new [] { "EducationalEnvironmentDescriptor" });
            }

            // InstructionLanguageDescriptor (string) maxLength
            if(this.InstructionLanguageDescriptor != null && this.InstructionLanguageDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for InstructionLanguageDescriptor, length must be less than 306.", new [] { "InstructionLanguageDescriptor" });
            }

            // MediumOfInstructionDescriptor (string) maxLength
            if(this.MediumOfInstructionDescriptor != null && this.MediumOfInstructionDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for MediumOfInstructionDescriptor, length must be less than 306.", new [] { "MediumOfInstructionDescriptor" });
            }

            // PopulationServedDescriptor (string) maxLength
            if(this.PopulationServedDescriptor != null && this.PopulationServedDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for PopulationServedDescriptor, length must be less than 306.", new [] { "PopulationServedDescriptor" });
            }

            // SectionName (string) maxLength
            if(this.SectionName != null && this.SectionName.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SectionName, length must be less than 100.", new [] { "SectionName" });
            }

            yield break;
        }
    }

}
