/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiStudentAcademicRecordDiploma
    /// </summary>
    [DataContract]
    public partial class EdFiStudentAcademicRecordDiploma :  IEquatable<EdFiStudentAcademicRecordDiploma>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentAcademicRecordDiploma" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiStudentAcademicRecordDiploma() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentAcademicRecordDiploma" /> class.
        /// </summary>
        /// <param name="DiplomaTypeDescriptor">The type of diploma/credential that is awarded to a student in recognition of his/her completion of the curricular requirements. (required).</param>
        /// <param name="DiplomaAwardDate">The month, day, and year on which the student met  graduation requirements and was awarded a diploma. (required).</param>
        /// <param name="AchievementCategoryDescriptor">The category of achievement attributed to the learner..</param>
        /// <param name="DiplomaLevelDescriptor">The level of diploma/credential that is awarded to a student in recognition of his/her completion of the curricular requirements.          Minimum high school program          Recommended high school program          Distinguished Achievement Program..</param>
        /// <param name="AchievementCategorySystem">The system that defines the categories by which an achievement is attributed to the learner..</param>
        /// <param name="AchievementTitle">The title assigned to the achievement..</param>
        /// <param name="Criteria">The criteria for competency-based completion of the achievement/award..</param>
        /// <param name="CriteriaURL">The Uniform Resource Locator (URL) for the unique address of a web page describing the competency-based completion criteria for the achievement/award..</param>
        /// <param name="CteCompleter">Indicated a student who reached a state-defined threshold of vocational education and who attained a high school diploma or its recognized state equivalent or GED..</param>
        /// <param name="DiplomaAwardExpiresDate">Date on which the award expires..</param>
        /// <param name="DiplomaDescription">The description of diploma given to the student for accomplishments..</param>
        /// <param name="EvidenceStatement">A statement or reference describing the evidence that the learner met the criteria for attainment of the Achievement..</param>
        /// <param name="ImageURL">The Uniform Resource Locator (URL) for the unique address of an image representing an award or badge associated with the Achievement..</param>
        /// <param name="IssuerName">The name of the agent, entity, or institution issuing the element..</param>
        /// <param name="IssuerOriginURL">The Uniform Resource Locator (URL) from which the award was issued..</param>
        public EdFiStudentAcademicRecordDiploma(string DiplomaTypeDescriptor = default(string), DateTime? DiplomaAwardDate = default(DateTime?), string AchievementCategoryDescriptor = default(string), string DiplomaLevelDescriptor = default(string), string AchievementCategorySystem = default(string), string AchievementTitle = default(string), string Criteria = default(string), string CriteriaURL = default(string), bool? CteCompleter = default(bool?), DateTime? DiplomaAwardExpiresDate = default(DateTime?), string DiplomaDescription = default(string), string EvidenceStatement = default(string), string ImageURL = default(string), string IssuerName = default(string), string IssuerOriginURL = default(string))
        {
            // to ensure "DiplomaTypeDescriptor" is required (not null)
            if (DiplomaTypeDescriptor == null)
            {
                throw new InvalidDataException("DiplomaTypeDescriptor is a required property for EdFiStudentAcademicRecordDiploma and cannot be null");
            }
            else
            {
                this.DiplomaTypeDescriptor = DiplomaTypeDescriptor;
            }
            // to ensure "DiplomaAwardDate" is required (not null)
            if (DiplomaAwardDate == null)
            {
                throw new InvalidDataException("DiplomaAwardDate is a required property for EdFiStudentAcademicRecordDiploma and cannot be null");
            }
            else
            {
                this.DiplomaAwardDate = DiplomaAwardDate;
            }
            this.AchievementCategoryDescriptor = AchievementCategoryDescriptor;
            this.DiplomaLevelDescriptor = DiplomaLevelDescriptor;
            this.AchievementCategorySystem = AchievementCategorySystem;
            this.AchievementTitle = AchievementTitle;
            this.Criteria = Criteria;
            this.CriteriaURL = CriteriaURL;
            this.CteCompleter = CteCompleter;
            this.DiplomaAwardExpiresDate = DiplomaAwardExpiresDate;
            this.DiplomaDescription = DiplomaDescription;
            this.EvidenceStatement = EvidenceStatement;
            this.ImageURL = ImageURL;
            this.IssuerName = IssuerName;
            this.IssuerOriginURL = IssuerOriginURL;
        }
        
        /// <summary>
        /// The type of diploma/credential that is awarded to a student in recognition of his/her completion of the curricular requirements.
        /// </summary>
        /// <value>The type of diploma/credential that is awarded to a student in recognition of his/her completion of the curricular requirements.</value>
        [DataMember(Name="diplomaTypeDescriptor", EmitDefaultValue=false)]
        public string DiplomaTypeDescriptor { get; set; }

        /// <summary>
        /// The month, day, and year on which the student met  graduation requirements and was awarded a diploma.
        /// </summary>
        /// <value>The month, day, and year on which the student met  graduation requirements and was awarded a diploma.</value>
        [DataMember(Name="diplomaAwardDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? DiplomaAwardDate { get; set; }

        /// <summary>
        /// The category of achievement attributed to the learner.
        /// </summary>
        /// <value>The category of achievement attributed to the learner.</value>
        [DataMember(Name="achievementCategoryDescriptor", EmitDefaultValue=false)]
        public string AchievementCategoryDescriptor { get; set; }

        /// <summary>
        /// The level of diploma/credential that is awarded to a student in recognition of his/her completion of the curricular requirements.          Minimum high school program          Recommended high school program          Distinguished Achievement Program.
        /// </summary>
        /// <value>The level of diploma/credential that is awarded to a student in recognition of his/her completion of the curricular requirements.          Minimum high school program          Recommended high school program          Distinguished Achievement Program.</value>
        [DataMember(Name="diplomaLevelDescriptor", EmitDefaultValue=false)]
        public string DiplomaLevelDescriptor { get; set; }

        /// <summary>
        /// The system that defines the categories by which an achievement is attributed to the learner.
        /// </summary>
        /// <value>The system that defines the categories by which an achievement is attributed to the learner.</value>
        [DataMember(Name="achievementCategorySystem", EmitDefaultValue=false)]
        public string AchievementCategorySystem { get; set; }

        /// <summary>
        /// The title assigned to the achievement.
        /// </summary>
        /// <value>The title assigned to the achievement.</value>
        [DataMember(Name="achievementTitle", EmitDefaultValue=false)]
        public string AchievementTitle { get; set; }

        /// <summary>
        /// The criteria for competency-based completion of the achievement/award.
        /// </summary>
        /// <value>The criteria for competency-based completion of the achievement/award.</value>
        [DataMember(Name="criteria", EmitDefaultValue=false)]
        public string Criteria { get; set; }

        /// <summary>
        /// The Uniform Resource Locator (URL) for the unique address of a web page describing the competency-based completion criteria for the achievement/award.
        /// </summary>
        /// <value>The Uniform Resource Locator (URL) for the unique address of a web page describing the competency-based completion criteria for the achievement/award.</value>
        [DataMember(Name="criteriaURL", EmitDefaultValue=false)]
        public string CriteriaURL { get; set; }

        /// <summary>
        /// Indicated a student who reached a state-defined threshold of vocational education and who attained a high school diploma or its recognized state equivalent or GED.
        /// </summary>
        /// <value>Indicated a student who reached a state-defined threshold of vocational education and who attained a high school diploma or its recognized state equivalent or GED.</value>
        [DataMember(Name="cteCompleter", EmitDefaultValue=false)]
        public bool? CteCompleter { get; set; }

        /// <summary>
        /// Date on which the award expires.
        /// </summary>
        /// <value>Date on which the award expires.</value>
        [DataMember(Name="diplomaAwardExpiresDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? DiplomaAwardExpiresDate { get; set; }

        /// <summary>
        /// The description of diploma given to the student for accomplishments.
        /// </summary>
        /// <value>The description of diploma given to the student for accomplishments.</value>
        [DataMember(Name="diplomaDescription", EmitDefaultValue=false)]
        public string DiplomaDescription { get; set; }

        /// <summary>
        /// A statement or reference describing the evidence that the learner met the criteria for attainment of the Achievement.
        /// </summary>
        /// <value>A statement or reference describing the evidence that the learner met the criteria for attainment of the Achievement.</value>
        [DataMember(Name="evidenceStatement", EmitDefaultValue=false)]
        public string EvidenceStatement { get; set; }

        /// <summary>
        /// The Uniform Resource Locator (URL) for the unique address of an image representing an award or badge associated with the Achievement.
        /// </summary>
        /// <value>The Uniform Resource Locator (URL) for the unique address of an image representing an award or badge associated with the Achievement.</value>
        [DataMember(Name="imageURL", EmitDefaultValue=false)]
        public string ImageURL { get; set; }

        /// <summary>
        /// The name of the agent, entity, or institution issuing the element.
        /// </summary>
        /// <value>The name of the agent, entity, or institution issuing the element.</value>
        [DataMember(Name="issuerName", EmitDefaultValue=false)]
        public string IssuerName { get; set; }

        /// <summary>
        /// The Uniform Resource Locator (URL) from which the award was issued.
        /// </summary>
        /// <value>The Uniform Resource Locator (URL) from which the award was issued.</value>
        [DataMember(Name="issuerOriginURL", EmitDefaultValue=false)]
        public string IssuerOriginURL { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiStudentAcademicRecordDiploma {\n");
            sb.Append("  DiplomaTypeDescriptor: ").Append(DiplomaTypeDescriptor).Append("\n");
            sb.Append("  DiplomaAwardDate: ").Append(DiplomaAwardDate).Append("\n");
            sb.Append("  AchievementCategoryDescriptor: ").Append(AchievementCategoryDescriptor).Append("\n");
            sb.Append("  DiplomaLevelDescriptor: ").Append(DiplomaLevelDescriptor).Append("\n");
            sb.Append("  AchievementCategorySystem: ").Append(AchievementCategorySystem).Append("\n");
            sb.Append("  AchievementTitle: ").Append(AchievementTitle).Append("\n");
            sb.Append("  Criteria: ").Append(Criteria).Append("\n");
            sb.Append("  CriteriaURL: ").Append(CriteriaURL).Append("\n");
            sb.Append("  CteCompleter: ").Append(CteCompleter).Append("\n");
            sb.Append("  DiplomaAwardExpiresDate: ").Append(DiplomaAwardExpiresDate).Append("\n");
            sb.Append("  DiplomaDescription: ").Append(DiplomaDescription).Append("\n");
            sb.Append("  EvidenceStatement: ").Append(EvidenceStatement).Append("\n");
            sb.Append("  ImageURL: ").Append(ImageURL).Append("\n");
            sb.Append("  IssuerName: ").Append(IssuerName).Append("\n");
            sb.Append("  IssuerOriginURL: ").Append(IssuerOriginURL).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiStudentAcademicRecordDiploma);
        }

        /// <summary>
        /// Returns true if EdFiStudentAcademicRecordDiploma instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiStudentAcademicRecordDiploma to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiStudentAcademicRecordDiploma input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.DiplomaTypeDescriptor == input.DiplomaTypeDescriptor ||
                    (this.DiplomaTypeDescriptor != null &&
                    this.DiplomaTypeDescriptor.Equals(input.DiplomaTypeDescriptor))
                ) && 
                (
                    this.DiplomaAwardDate == input.DiplomaAwardDate ||
                    (this.DiplomaAwardDate != null &&
                    this.DiplomaAwardDate.Equals(input.DiplomaAwardDate))
                ) && 
                (
                    this.AchievementCategoryDescriptor == input.AchievementCategoryDescriptor ||
                    (this.AchievementCategoryDescriptor != null &&
                    this.AchievementCategoryDescriptor.Equals(input.AchievementCategoryDescriptor))
                ) && 
                (
                    this.DiplomaLevelDescriptor == input.DiplomaLevelDescriptor ||
                    (this.DiplomaLevelDescriptor != null &&
                    this.DiplomaLevelDescriptor.Equals(input.DiplomaLevelDescriptor))
                ) && 
                (
                    this.AchievementCategorySystem == input.AchievementCategorySystem ||
                    (this.AchievementCategorySystem != null &&
                    this.AchievementCategorySystem.Equals(input.AchievementCategorySystem))
                ) && 
                (
                    this.AchievementTitle == input.AchievementTitle ||
                    (this.AchievementTitle != null &&
                    this.AchievementTitle.Equals(input.AchievementTitle))
                ) && 
                (
                    this.Criteria == input.Criteria ||
                    (this.Criteria != null &&
                    this.Criteria.Equals(input.Criteria))
                ) && 
                (
                    this.CriteriaURL == input.CriteriaURL ||
                    (this.CriteriaURL != null &&
                    this.CriteriaURL.Equals(input.CriteriaURL))
                ) && 
                (
                    this.CteCompleter == input.CteCompleter ||
                    (this.CteCompleter != null &&
                    this.CteCompleter.Equals(input.CteCompleter))
                ) && 
                (
                    this.DiplomaAwardExpiresDate == input.DiplomaAwardExpiresDate ||
                    (this.DiplomaAwardExpiresDate != null &&
                    this.DiplomaAwardExpiresDate.Equals(input.DiplomaAwardExpiresDate))
                ) && 
                (
                    this.DiplomaDescription == input.DiplomaDescription ||
                    (this.DiplomaDescription != null &&
                    this.DiplomaDescription.Equals(input.DiplomaDescription))
                ) && 
                (
                    this.EvidenceStatement == input.EvidenceStatement ||
                    (this.EvidenceStatement != null &&
                    this.EvidenceStatement.Equals(input.EvidenceStatement))
                ) && 
                (
                    this.ImageURL == input.ImageURL ||
                    (this.ImageURL != null &&
                    this.ImageURL.Equals(input.ImageURL))
                ) && 
                (
                    this.IssuerName == input.IssuerName ||
                    (this.IssuerName != null &&
                    this.IssuerName.Equals(input.IssuerName))
                ) && 
                (
                    this.IssuerOriginURL == input.IssuerOriginURL ||
                    (this.IssuerOriginURL != null &&
                    this.IssuerOriginURL.Equals(input.IssuerOriginURL))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.DiplomaTypeDescriptor != null)
                    hashCode = hashCode * 59 + this.DiplomaTypeDescriptor.GetHashCode();
                if (this.DiplomaAwardDate != null)
                    hashCode = hashCode * 59 + this.DiplomaAwardDate.GetHashCode();
                if (this.AchievementCategoryDescriptor != null)
                    hashCode = hashCode * 59 + this.AchievementCategoryDescriptor.GetHashCode();
                if (this.DiplomaLevelDescriptor != null)
                    hashCode = hashCode * 59 + this.DiplomaLevelDescriptor.GetHashCode();
                if (this.AchievementCategorySystem != null)
                    hashCode = hashCode * 59 + this.AchievementCategorySystem.GetHashCode();
                if (this.AchievementTitle != null)
                    hashCode = hashCode * 59 + this.AchievementTitle.GetHashCode();
                if (this.Criteria != null)
                    hashCode = hashCode * 59 + this.Criteria.GetHashCode();
                if (this.CriteriaURL != null)
                    hashCode = hashCode * 59 + this.CriteriaURL.GetHashCode();
                if (this.CteCompleter != null)
                    hashCode = hashCode * 59 + this.CteCompleter.GetHashCode();
                if (this.DiplomaAwardExpiresDate != null)
                    hashCode = hashCode * 59 + this.DiplomaAwardExpiresDate.GetHashCode();
                if (this.DiplomaDescription != null)
                    hashCode = hashCode * 59 + this.DiplomaDescription.GetHashCode();
                if (this.EvidenceStatement != null)
                    hashCode = hashCode * 59 + this.EvidenceStatement.GetHashCode();
                if (this.ImageURL != null)
                    hashCode = hashCode * 59 + this.ImageURL.GetHashCode();
                if (this.IssuerName != null)
                    hashCode = hashCode * 59 + this.IssuerName.GetHashCode();
                if (this.IssuerOriginURL != null)
                    hashCode = hashCode * 59 + this.IssuerOriginURL.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // DiplomaTypeDescriptor (string) maxLength
            if(this.DiplomaTypeDescriptor != null && this.DiplomaTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DiplomaTypeDescriptor, length must be less than 306.", new [] { "DiplomaTypeDescriptor" });
            }

            // AchievementCategoryDescriptor (string) maxLength
            if(this.AchievementCategoryDescriptor != null && this.AchievementCategoryDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AchievementCategoryDescriptor, length must be less than 306.", new [] { "AchievementCategoryDescriptor" });
            }

            // DiplomaLevelDescriptor (string) maxLength
            if(this.DiplomaLevelDescriptor != null && this.DiplomaLevelDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DiplomaLevelDescriptor, length must be less than 306.", new [] { "DiplomaLevelDescriptor" });
            }

            // AchievementCategorySystem (string) maxLength
            if(this.AchievementCategorySystem != null && this.AchievementCategorySystem.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AchievementCategorySystem, length must be less than 60.", new [] { "AchievementCategorySystem" });
            }

            // AchievementTitle (string) maxLength
            if(this.AchievementTitle != null && this.AchievementTitle.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AchievementTitle, length must be less than 60.", new [] { "AchievementTitle" });
            }

            // Criteria (string) maxLength
            if(this.Criteria != null && this.Criteria.Length > 150)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Criteria, length must be less than 150.", new [] { "Criteria" });
            }

            // CriteriaURL (string) maxLength
            if(this.CriteriaURL != null && this.CriteriaURL.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CriteriaURL, length must be less than 255.", new [] { "CriteriaURL" });
            }

            // DiplomaDescription (string) maxLength
            if(this.DiplomaDescription != null && this.DiplomaDescription.Length > 80)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DiplomaDescription, length must be less than 80.", new [] { "DiplomaDescription" });
            }

            // EvidenceStatement (string) maxLength
            if(this.EvidenceStatement != null && this.EvidenceStatement.Length > 150)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for EvidenceStatement, length must be less than 150.", new [] { "EvidenceStatement" });
            }

            // ImageURL (string) maxLength
            if(this.ImageURL != null && this.ImageURL.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ImageURL, length must be less than 255.", new [] { "ImageURL" });
            }

            // IssuerName (string) maxLength
            if(this.IssuerName != null && this.IssuerName.Length > 150)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for IssuerName, length must be less than 150.", new [] { "IssuerName" });
            }

            // IssuerOriginURL (string) maxLength
            if(this.IssuerOriginURL != null && this.IssuerOriginURL.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for IssuerOriginURL, length must be less than 255.", new [] { "IssuerOriginURL" });
            }

            yield break;
        }
    }

}
