/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiStudentCTEProgramAssociation
    /// </summary>
    [DataContract]
    public partial class EdFiStudentCTEProgramAssociation :  IEquatable<EdFiStudentCTEProgramAssociation>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentCTEProgramAssociation" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiStudentCTEProgramAssociation() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentCTEProgramAssociation" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="BeginDate">The earliest date the student is involved with the program. Typically, this is the date the student becomes eligible for the program. (required).</param>
        /// <param name="EducationOrganizationReference">EducationOrganizationReference (required).</param>
        /// <param name="ProgramReference">ProgramReference (required).</param>
        /// <param name="StudentReference">StudentReference (required).</param>
        /// <param name="CtePrograms">An unordered collection of studentCTEProgramAssociationCTEPrograms. The career cluster representing the career path of the Vocational/Career Tech concentrator..</param>
        /// <param name="CteProgramServices">An unordered collection of studentCTEProgramAssociationCTEProgramServices. Indicates the Service(s) being provided to the Student by the CTE Program..</param>
        /// <param name="EndDate">The month, day, and year on which the Student exited the Program or stopped receiving services..</param>
        /// <param name="NonTraditionalGenderStatus">Indicator that student is from a gender group that comprises less than 25% of the individuals employed in an occupation or field of work..</param>
        /// <param name="ParticipationStatus">ParticipationStatus.</param>
        /// <param name="PrivateCTEProgram">Indicator that student participated in career and technical education at private agencies or institutions that are reported by the state for purposes of the Elementary and Secondary Education Act (ESEA).  Students in private institutions which do not receive Perkins funding are reported only in the state file..</param>
        /// <param name="ReasonExitedDescriptor">The reason the child left the Program within a school or district..</param>
        /// <param name="ServedOutsideOfRegularSession">Indicates whether the Student received services during the summer session or between sessions..</param>
        /// <param name="Services">An unordered collection of studentCTEProgramAssociationServices. Indicates the Service(s) being provided to the Student by the Program..</param>
        /// <param name="TechnicalSkillsAssessmentDescriptor">Results of technical skills assessment aligned with industry recognized standards..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        /// <param name="Ext">Ext.</param>
        public EdFiStudentCTEProgramAssociation(string Id = default(string), DateTime? BeginDate = default(DateTime?), EdFiEducationOrganizationReference EducationOrganizationReference = default(EdFiEducationOrganizationReference), EdFiProgramReference ProgramReference = default(EdFiProgramReference), EdFiStudentReference StudentReference = default(EdFiStudentReference), List<EdFiStudentCTEProgramAssociationCTEProgram> CtePrograms = default(List<EdFiStudentCTEProgramAssociationCTEProgram>), List<EdFiStudentCTEProgramAssociationCTEProgramService> CteProgramServices = default(List<EdFiStudentCTEProgramAssociationCTEProgramService>), DateTime? EndDate = default(DateTime?), bool? NonTraditionalGenderStatus = default(bool?), EdFiGeneralStudentProgramAssociationParticipationStatus ParticipationStatus = default(EdFiGeneralStudentProgramAssociationParticipationStatus), bool? PrivateCTEProgram = default(bool?), string ReasonExitedDescriptor = default(string), bool? ServedOutsideOfRegularSession = default(bool?), List<EdFiStudentCTEProgramAssociationService> Services = default(List<EdFiStudentCTEProgramAssociationService>), string TechnicalSkillsAssessmentDescriptor = default(string), string Etag = default(string), StudentCTEProgramAssociationExtensions Ext = default(StudentCTEProgramAssociationExtensions))
        {
            // to ensure "BeginDate" is required (not null)
            if (BeginDate == null)
            {
                throw new InvalidDataException("BeginDate is a required property for EdFiStudentCTEProgramAssociation and cannot be null");
            }
            else
            {
                this.BeginDate = BeginDate;
            }
            // to ensure "EducationOrganizationReference" is required (not null)
            if (EducationOrganizationReference == null)
            {
                throw new InvalidDataException("EducationOrganizationReference is a required property for EdFiStudentCTEProgramAssociation and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = EducationOrganizationReference;
            }
            // to ensure "ProgramReference" is required (not null)
            if (ProgramReference == null)
            {
                throw new InvalidDataException("ProgramReference is a required property for EdFiStudentCTEProgramAssociation and cannot be null");
            }
            else
            {
                this.ProgramReference = ProgramReference;
            }
            // to ensure "StudentReference" is required (not null)
            if (StudentReference == null)
            {
                throw new InvalidDataException("StudentReference is a required property for EdFiStudentCTEProgramAssociation and cannot be null");
            }
            else
            {
                this.StudentReference = StudentReference;
            }
            this.Id = Id;
            this.CtePrograms = CtePrograms;
            this.CteProgramServices = CteProgramServices;
            this.EndDate = EndDate;
            this.NonTraditionalGenderStatus = NonTraditionalGenderStatus;
            this.ParticipationStatus = ParticipationStatus;
            this.PrivateCTEProgram = PrivateCTEProgram;
            this.ReasonExitedDescriptor = ReasonExitedDescriptor;
            this.ServedOutsideOfRegularSession = ServedOutsideOfRegularSession;
            this.Services = Services;
            this.TechnicalSkillsAssessmentDescriptor = TechnicalSkillsAssessmentDescriptor;
            this.Etag = Etag;
            this.Ext = Ext;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The earliest date the student is involved with the program. Typically, this is the date the student becomes eligible for the program.
        /// </summary>
        /// <value>The earliest date the student is involved with the program. Typically, this is the date the student becomes eligible for the program.</value>
        [DataMember(Name="beginDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? BeginDate { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// Gets or Sets ProgramReference
        /// </summary>
        [DataMember(Name="programReference", EmitDefaultValue=false)]
        public EdFiProgramReference ProgramReference { get; set; }

        /// <summary>
        /// Gets or Sets StudentReference
        /// </summary>
        [DataMember(Name="studentReference", EmitDefaultValue=false)]
        public EdFiStudentReference StudentReference { get; set; }

        /// <summary>
        /// An unordered collection of studentCTEProgramAssociationCTEPrograms. The career cluster representing the career path of the Vocational/Career Tech concentrator.
        /// </summary>
        /// <value>An unordered collection of studentCTEProgramAssociationCTEPrograms. The career cluster representing the career path of the Vocational/Career Tech concentrator.</value>
        [DataMember(Name="ctePrograms", EmitDefaultValue=false)]
        public List<EdFiStudentCTEProgramAssociationCTEProgram> CtePrograms { get; set; }

        /// <summary>
        /// An unordered collection of studentCTEProgramAssociationCTEProgramServices. Indicates the Service(s) being provided to the Student by the CTE Program.
        /// </summary>
        /// <value>An unordered collection of studentCTEProgramAssociationCTEProgramServices. Indicates the Service(s) being provided to the Student by the CTE Program.</value>
        [DataMember(Name="cteProgramServices", EmitDefaultValue=false)]
        public List<EdFiStudentCTEProgramAssociationCTEProgramService> CteProgramServices { get; set; }

        /// <summary>
        /// The month, day, and year on which the Student exited the Program or stopped receiving services.
        /// </summary>
        /// <value>The month, day, and year on which the Student exited the Program or stopped receiving services.</value>
        [DataMember(Name="endDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? EndDate { get; set; }

        /// <summary>
        /// Indicator that student is from a gender group that comprises less than 25% of the individuals employed in an occupation or field of work.
        /// </summary>
        /// <value>Indicator that student is from a gender group that comprises less than 25% of the individuals employed in an occupation or field of work.</value>
        [DataMember(Name="nonTraditionalGenderStatus", EmitDefaultValue=false)]
        public bool? NonTraditionalGenderStatus { get; set; }

        /// <summary>
        /// Gets or Sets ParticipationStatus
        /// </summary>
        [DataMember(Name="participationStatus", EmitDefaultValue=false)]
        public EdFiGeneralStudentProgramAssociationParticipationStatus ParticipationStatus { get; set; }

        /// <summary>
        /// Indicator that student participated in career and technical education at private agencies or institutions that are reported by the state for purposes of the Elementary and Secondary Education Act (ESEA).  Students in private institutions which do not receive Perkins funding are reported only in the state file.
        /// </summary>
        /// <value>Indicator that student participated in career and technical education at private agencies or institutions that are reported by the state for purposes of the Elementary and Secondary Education Act (ESEA).  Students in private institutions which do not receive Perkins funding are reported only in the state file.</value>
        [DataMember(Name="privateCTEProgram", EmitDefaultValue=false)]
        public bool? PrivateCTEProgram { get; set; }

        /// <summary>
        /// The reason the child left the Program within a school or district.
        /// </summary>
        /// <value>The reason the child left the Program within a school or district.</value>
        [DataMember(Name="reasonExitedDescriptor", EmitDefaultValue=false)]
        public string ReasonExitedDescriptor { get; set; }

        /// <summary>
        /// Indicates whether the Student received services during the summer session or between sessions.
        /// </summary>
        /// <value>Indicates whether the Student received services during the summer session or between sessions.</value>
        [DataMember(Name="servedOutsideOfRegularSession", EmitDefaultValue=false)]
        public bool? ServedOutsideOfRegularSession { get; set; }

        /// <summary>
        /// An unordered collection of studentCTEProgramAssociationServices. Indicates the Service(s) being provided to the Student by the Program.
        /// </summary>
        /// <value>An unordered collection of studentCTEProgramAssociationServices. Indicates the Service(s) being provided to the Student by the Program.</value>
        [DataMember(Name="services", EmitDefaultValue=false)]
        public List<EdFiStudentCTEProgramAssociationService> Services { get; set; }

        /// <summary>
        /// Results of technical skills assessment aligned with industry recognized standards.
        /// </summary>
        /// <value>Results of technical skills assessment aligned with industry recognized standards.</value>
        [DataMember(Name="technicalSkillsAssessmentDescriptor", EmitDefaultValue=false)]
        public string TechnicalSkillsAssessmentDescriptor { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Gets or Sets Ext
        /// </summary>
        [DataMember(Name="_ext", EmitDefaultValue=false)]
        public StudentCTEProgramAssociationExtensions Ext { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiStudentCTEProgramAssociation {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  BeginDate: ").Append(BeginDate).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  ProgramReference: ").Append(ProgramReference).Append("\n");
            sb.Append("  StudentReference: ").Append(StudentReference).Append("\n");
            sb.Append("  CtePrograms: ").Append(CtePrograms).Append("\n");
            sb.Append("  CteProgramServices: ").Append(CteProgramServices).Append("\n");
            sb.Append("  EndDate: ").Append(EndDate).Append("\n");
            sb.Append("  NonTraditionalGenderStatus: ").Append(NonTraditionalGenderStatus).Append("\n");
            sb.Append("  ParticipationStatus: ").Append(ParticipationStatus).Append("\n");
            sb.Append("  PrivateCTEProgram: ").Append(PrivateCTEProgram).Append("\n");
            sb.Append("  ReasonExitedDescriptor: ").Append(ReasonExitedDescriptor).Append("\n");
            sb.Append("  ServedOutsideOfRegularSession: ").Append(ServedOutsideOfRegularSession).Append("\n");
            sb.Append("  Services: ").Append(Services).Append("\n");
            sb.Append("  TechnicalSkillsAssessmentDescriptor: ").Append(TechnicalSkillsAssessmentDescriptor).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("  Ext: ").Append(Ext).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiStudentCTEProgramAssociation);
        }

        /// <summary>
        /// Returns true if EdFiStudentCTEProgramAssociation instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiStudentCTEProgramAssociation to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiStudentCTEProgramAssociation input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.BeginDate == input.BeginDate ||
                    (this.BeginDate != null &&
                    this.BeginDate.Equals(input.BeginDate))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.ProgramReference == input.ProgramReference ||
                    (this.ProgramReference != null &&
                    this.ProgramReference.Equals(input.ProgramReference))
                ) && 
                (
                    this.StudentReference == input.StudentReference ||
                    (this.StudentReference != null &&
                    this.StudentReference.Equals(input.StudentReference))
                ) && 
                (
                    this.CtePrograms == input.CtePrograms ||
                    this.CtePrograms != null &&
                    this.CtePrograms.SequenceEqual(input.CtePrograms)
                ) && 
                (
                    this.CteProgramServices == input.CteProgramServices ||
                    this.CteProgramServices != null &&
                    this.CteProgramServices.SequenceEqual(input.CteProgramServices)
                ) && 
                (
                    this.EndDate == input.EndDate ||
                    (this.EndDate != null &&
                    this.EndDate.Equals(input.EndDate))
                ) && 
                (
                    this.NonTraditionalGenderStatus == input.NonTraditionalGenderStatus ||
                    (this.NonTraditionalGenderStatus != null &&
                    this.NonTraditionalGenderStatus.Equals(input.NonTraditionalGenderStatus))
                ) && 
                (
                    this.ParticipationStatus == input.ParticipationStatus ||
                    (this.ParticipationStatus != null &&
                    this.ParticipationStatus.Equals(input.ParticipationStatus))
                ) && 
                (
                    this.PrivateCTEProgram == input.PrivateCTEProgram ||
                    (this.PrivateCTEProgram != null &&
                    this.PrivateCTEProgram.Equals(input.PrivateCTEProgram))
                ) && 
                (
                    this.ReasonExitedDescriptor == input.ReasonExitedDescriptor ||
                    (this.ReasonExitedDescriptor != null &&
                    this.ReasonExitedDescriptor.Equals(input.ReasonExitedDescriptor))
                ) && 
                (
                    this.ServedOutsideOfRegularSession == input.ServedOutsideOfRegularSession ||
                    (this.ServedOutsideOfRegularSession != null &&
                    this.ServedOutsideOfRegularSession.Equals(input.ServedOutsideOfRegularSession))
                ) && 
                (
                    this.Services == input.Services ||
                    this.Services != null &&
                    this.Services.SequenceEqual(input.Services)
                ) && 
                (
                    this.TechnicalSkillsAssessmentDescriptor == input.TechnicalSkillsAssessmentDescriptor ||
                    (this.TechnicalSkillsAssessmentDescriptor != null &&
                    this.TechnicalSkillsAssessmentDescriptor.Equals(input.TechnicalSkillsAssessmentDescriptor))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                ) && 
                (
                    this.Ext == input.Ext ||
                    (this.Ext != null &&
                    this.Ext.Equals(input.Ext))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.BeginDate != null)
                    hashCode = hashCode * 59 + this.BeginDate.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.ProgramReference != null)
                    hashCode = hashCode * 59 + this.ProgramReference.GetHashCode();
                if (this.StudentReference != null)
                    hashCode = hashCode * 59 + this.StudentReference.GetHashCode();
                if (this.CtePrograms != null)
                    hashCode = hashCode * 59 + this.CtePrograms.GetHashCode();
                if (this.CteProgramServices != null)
                    hashCode = hashCode * 59 + this.CteProgramServices.GetHashCode();
                if (this.EndDate != null)
                    hashCode = hashCode * 59 + this.EndDate.GetHashCode();
                if (this.NonTraditionalGenderStatus != null)
                    hashCode = hashCode * 59 + this.NonTraditionalGenderStatus.GetHashCode();
                if (this.ParticipationStatus != null)
                    hashCode = hashCode * 59 + this.ParticipationStatus.GetHashCode();
                if (this.PrivateCTEProgram != null)
                    hashCode = hashCode * 59 + this.PrivateCTEProgram.GetHashCode();
                if (this.ReasonExitedDescriptor != null)
                    hashCode = hashCode * 59 + this.ReasonExitedDescriptor.GetHashCode();
                if (this.ServedOutsideOfRegularSession != null)
                    hashCode = hashCode * 59 + this.ServedOutsideOfRegularSession.GetHashCode();
                if (this.Services != null)
                    hashCode = hashCode * 59 + this.Services.GetHashCode();
                if (this.TechnicalSkillsAssessmentDescriptor != null)
                    hashCode = hashCode * 59 + this.TechnicalSkillsAssessmentDescriptor.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                if (this.Ext != null)
                    hashCode = hashCode * 59 + this.Ext.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // ReasonExitedDescriptor (string) maxLength
            if(this.ReasonExitedDescriptor != null && this.ReasonExitedDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ReasonExitedDescriptor, length must be less than 306.", new [] { "ReasonExitedDescriptor" });
            }

            // TechnicalSkillsAssessmentDescriptor (string) maxLength
            if(this.TechnicalSkillsAssessmentDescriptor != null && this.TechnicalSkillsAssessmentDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for TechnicalSkillsAssessmentDescriptor, length must be less than 306.", new [] { "TechnicalSkillsAssessmentDescriptor" });
            }

            yield break;
        }
    }

}
