/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiSurveyQuestion
    /// </summary>
    [DataContract]
    public partial class EdFiSurveyQuestion :  IEquatable<EdFiSurveyQuestion>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSurveyQuestion" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiSurveyQuestion() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSurveyQuestion" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="QuestionCode">The identifying code for the question, unique for the survey. (required).</param>
        /// <param name="SurveyReference">SurveyReference (required).</param>
        /// <param name="SurveySectionReference">SurveySectionReference.</param>
        /// <param name="Matrices">An unordered collection of surveyQuestionMatrices. Information about the matrix element in the survey..</param>
        /// <param name="QuestionFormDescriptor">The form or type of question. (required).</param>
        /// <param name="QuestionText">The text of the question. (required).</param>
        /// <param name="ResponseChoices">An unordered collection of surveyQuestionResponseChoices. The optional list of possible responses to a survey question..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiSurveyQuestion(string Id = default(string), string QuestionCode = default(string), EdFiSurveyReference SurveyReference = default(EdFiSurveyReference), EdFiSurveySectionReference SurveySectionReference = default(EdFiSurveySectionReference), List<EdFiSurveyQuestionMatrix> Matrices = default(List<EdFiSurveyQuestionMatrix>), string QuestionFormDescriptor = default(string), string QuestionText = default(string), List<EdFiSurveyQuestionResponseChoice> ResponseChoices = default(List<EdFiSurveyQuestionResponseChoice>), string Etag = default(string))
        {
            // to ensure "QuestionCode" is required (not null)
            if (QuestionCode == null)
            {
                throw new InvalidDataException("QuestionCode is a required property for EdFiSurveyQuestion and cannot be null");
            }
            else
            {
                this.QuestionCode = QuestionCode;
            }
            // to ensure "SurveyReference" is required (not null)
            if (SurveyReference == null)
            {
                throw new InvalidDataException("SurveyReference is a required property for EdFiSurveyQuestion and cannot be null");
            }
            else
            {
                this.SurveyReference = SurveyReference;
            }
            // to ensure "QuestionFormDescriptor" is required (not null)
            if (QuestionFormDescriptor == null)
            {
                throw new InvalidDataException("QuestionFormDescriptor is a required property for EdFiSurveyQuestion and cannot be null");
            }
            else
            {
                this.QuestionFormDescriptor = QuestionFormDescriptor;
            }
            // to ensure "QuestionText" is required (not null)
            if (QuestionText == null)
            {
                throw new InvalidDataException("QuestionText is a required property for EdFiSurveyQuestion and cannot be null");
            }
            else
            {
                this.QuestionText = QuestionText;
            }
            this.Id = Id;
            this.SurveySectionReference = SurveySectionReference;
            this.Matrices = Matrices;
            this.ResponseChoices = ResponseChoices;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The identifying code for the question, unique for the survey.
        /// </summary>
        /// <value>The identifying code for the question, unique for the survey.</value>
        [DataMember(Name="questionCode", EmitDefaultValue=false)]
        public string QuestionCode { get; set; }

        /// <summary>
        /// Gets or Sets SurveyReference
        /// </summary>
        [DataMember(Name="surveyReference", EmitDefaultValue=false)]
        public EdFiSurveyReference SurveyReference { get; set; }

        /// <summary>
        /// Gets or Sets SurveySectionReference
        /// </summary>
        [DataMember(Name="surveySectionReference", EmitDefaultValue=false)]
        public EdFiSurveySectionReference SurveySectionReference { get; set; }

        /// <summary>
        /// An unordered collection of surveyQuestionMatrices. Information about the matrix element in the survey.
        /// </summary>
        /// <value>An unordered collection of surveyQuestionMatrices. Information about the matrix element in the survey.</value>
        [DataMember(Name="matrices", EmitDefaultValue=false)]
        public List<EdFiSurveyQuestionMatrix> Matrices { get; set; }

        /// <summary>
        /// The form or type of question.
        /// </summary>
        /// <value>The form or type of question.</value>
        [DataMember(Name="questionFormDescriptor", EmitDefaultValue=false)]
        public string QuestionFormDescriptor { get; set; }

        /// <summary>
        /// The text of the question.
        /// </summary>
        /// <value>The text of the question.</value>
        [DataMember(Name="questionText", EmitDefaultValue=false)]
        public string QuestionText { get; set; }

        /// <summary>
        /// An unordered collection of surveyQuestionResponseChoices. The optional list of possible responses to a survey question.
        /// </summary>
        /// <value>An unordered collection of surveyQuestionResponseChoices. The optional list of possible responses to a survey question.</value>
        [DataMember(Name="responseChoices", EmitDefaultValue=false)]
        public List<EdFiSurveyQuestionResponseChoice> ResponseChoices { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiSurveyQuestion {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  QuestionCode: ").Append(QuestionCode).Append("\n");
            sb.Append("  SurveyReference: ").Append(SurveyReference).Append("\n");
            sb.Append("  SurveySectionReference: ").Append(SurveySectionReference).Append("\n");
            sb.Append("  Matrices: ").Append(Matrices).Append("\n");
            sb.Append("  QuestionFormDescriptor: ").Append(QuestionFormDescriptor).Append("\n");
            sb.Append("  QuestionText: ").Append(QuestionText).Append("\n");
            sb.Append("  ResponseChoices: ").Append(ResponseChoices).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiSurveyQuestion);
        }

        /// <summary>
        /// Returns true if EdFiSurveyQuestion instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiSurveyQuestion to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiSurveyQuestion input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.QuestionCode == input.QuestionCode ||
                    (this.QuestionCode != null &&
                    this.QuestionCode.Equals(input.QuestionCode))
                ) && 
                (
                    this.SurveyReference == input.SurveyReference ||
                    (this.SurveyReference != null &&
                    this.SurveyReference.Equals(input.SurveyReference))
                ) && 
                (
                    this.SurveySectionReference == input.SurveySectionReference ||
                    (this.SurveySectionReference != null &&
                    this.SurveySectionReference.Equals(input.SurveySectionReference))
                ) && 
                (
                    this.Matrices == input.Matrices ||
                    this.Matrices != null &&
                    this.Matrices.SequenceEqual(input.Matrices)
                ) && 
                (
                    this.QuestionFormDescriptor == input.QuestionFormDescriptor ||
                    (this.QuestionFormDescriptor != null &&
                    this.QuestionFormDescriptor.Equals(input.QuestionFormDescriptor))
                ) && 
                (
                    this.QuestionText == input.QuestionText ||
                    (this.QuestionText != null &&
                    this.QuestionText.Equals(input.QuestionText))
                ) && 
                (
                    this.ResponseChoices == input.ResponseChoices ||
                    this.ResponseChoices != null &&
                    this.ResponseChoices.SequenceEqual(input.ResponseChoices)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.QuestionCode != null)
                    hashCode = hashCode * 59 + this.QuestionCode.GetHashCode();
                if (this.SurveyReference != null)
                    hashCode = hashCode * 59 + this.SurveyReference.GetHashCode();
                if (this.SurveySectionReference != null)
                    hashCode = hashCode * 59 + this.SurveySectionReference.GetHashCode();
                if (this.Matrices != null)
                    hashCode = hashCode * 59 + this.Matrices.GetHashCode();
                if (this.QuestionFormDescriptor != null)
                    hashCode = hashCode * 59 + this.QuestionFormDescriptor.GetHashCode();
                if (this.QuestionText != null)
                    hashCode = hashCode * 59 + this.QuestionText.GetHashCode();
                if (this.ResponseChoices != null)
                    hashCode = hashCode * 59 + this.ResponseChoices.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // QuestionCode (string) maxLength
            if(this.QuestionCode != null && this.QuestionCode.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for QuestionCode, length must be less than 60.", new [] { "QuestionCode" });
            }

            // QuestionFormDescriptor (string) maxLength
            if(this.QuestionFormDescriptor != null && this.QuestionFormDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for QuestionFormDescriptor, length must be less than 306.", new [] { "QuestionFormDescriptor" });
            }

            // QuestionText (string) maxLength
            if(this.QuestionText != null && this.QuestionText.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for QuestionText, length must be less than 1024.", new [] { "QuestionText" });
            }

            yield break;
        }
    }

}
