/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiSurveyResponse
    /// </summary>
    [DataContract]
    public partial class EdFiSurveyResponse :  IEquatable<EdFiSurveyResponse>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSurveyResponse" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiSurveyResponse() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSurveyResponse" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="SurveyResponseIdentifier">The identifier of the survey typically from the survey application. (required).</param>
        /// <param name="ParentReference">ParentReference.</param>
        /// <param name="StaffReference">StaffReference.</param>
        /// <param name="StudentReference">StudentReference.</param>
        /// <param name="SurveyReference">SurveyReference (required).</param>
        /// <param name="ElectronicMailAddress">Email address of the respondent..</param>
        /// <param name="FullName">Full name of the respondent..</param>
        /// <param name="Location">Location of the respondent, often a city, district, or school..</param>
        /// <param name="ResponseDate">Date of the survey response. (required).</param>
        /// <param name="ResponseTime">The amount of time (in seconds) it took for the respondent to complete the survey..</param>
        /// <param name="SurveyLevels">An unordered collection of surveyResponseSurveyLevels. Provides information about the respondents of a survey and how they can be grouped together..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiSurveyResponse(string Id = default(string), string SurveyResponseIdentifier = default(string), EdFiParentReference ParentReference = default(EdFiParentReference), EdFiStaffReference StaffReference = default(EdFiStaffReference), EdFiStudentReference StudentReference = default(EdFiStudentReference), EdFiSurveyReference SurveyReference = default(EdFiSurveyReference), string ElectronicMailAddress = default(string), string FullName = default(string), string Location = default(string), DateTime? ResponseDate = default(DateTime?), int? ResponseTime = default(int?), List<EdFiSurveyResponseSurveyLevel> SurveyLevels = default(List<EdFiSurveyResponseSurveyLevel>), string Etag = default(string))
        {
            // to ensure "SurveyResponseIdentifier" is required (not null)
            if (SurveyResponseIdentifier == null)
            {
                throw new InvalidDataException("SurveyResponseIdentifier is a required property for EdFiSurveyResponse and cannot be null");
            }
            else
            {
                this.SurveyResponseIdentifier = SurveyResponseIdentifier;
            }
            // to ensure "SurveyReference" is required (not null)
            if (SurveyReference == null)
            {
                throw new InvalidDataException("SurveyReference is a required property for EdFiSurveyResponse and cannot be null");
            }
            else
            {
                this.SurveyReference = SurveyReference;
            }
            // to ensure "ResponseDate" is required (not null)
            if (ResponseDate == null)
            {
                throw new InvalidDataException("ResponseDate is a required property for EdFiSurveyResponse and cannot be null");
            }
            else
            {
                this.ResponseDate = ResponseDate;
            }
            this.Id = Id;
            this.ParentReference = ParentReference;
            this.StaffReference = StaffReference;
            this.StudentReference = StudentReference;
            this.ElectronicMailAddress = ElectronicMailAddress;
            this.FullName = FullName;
            this.Location = Location;
            this.ResponseTime = ResponseTime;
            this.SurveyLevels = SurveyLevels;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The identifier of the survey typically from the survey application.
        /// </summary>
        /// <value>The identifier of the survey typically from the survey application.</value>
        [DataMember(Name="surveyResponseIdentifier", EmitDefaultValue=false)]
        public string SurveyResponseIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets ParentReference
        /// </summary>
        [DataMember(Name="parentReference", EmitDefaultValue=false)]
        public EdFiParentReference ParentReference { get; set; }

        /// <summary>
        /// Gets or Sets StaffReference
        /// </summary>
        [DataMember(Name="staffReference", EmitDefaultValue=false)]
        public EdFiStaffReference StaffReference { get; set; }

        /// <summary>
        /// Gets or Sets StudentReference
        /// </summary>
        [DataMember(Name="studentReference", EmitDefaultValue=false)]
        public EdFiStudentReference StudentReference { get; set; }

        /// <summary>
        /// Gets or Sets SurveyReference
        /// </summary>
        [DataMember(Name="surveyReference", EmitDefaultValue=false)]
        public EdFiSurveyReference SurveyReference { get; set; }

        /// <summary>
        /// Email address of the respondent.
        /// </summary>
        /// <value>Email address of the respondent.</value>
        [DataMember(Name="electronicMailAddress", EmitDefaultValue=false)]
        public string ElectronicMailAddress { get; set; }

        /// <summary>
        /// Full name of the respondent.
        /// </summary>
        /// <value>Full name of the respondent.</value>
        [DataMember(Name="fullName", EmitDefaultValue=false)]
        public string FullName { get; set; }

        /// <summary>
        /// Location of the respondent, often a city, district, or school.
        /// </summary>
        /// <value>Location of the respondent, often a city, district, or school.</value>
        [DataMember(Name="location", EmitDefaultValue=false)]
        public string Location { get; set; }

        /// <summary>
        /// Date of the survey response.
        /// </summary>
        /// <value>Date of the survey response.</value>
        [DataMember(Name="responseDate", EmitDefaultValue=false)]
        [JsonConverter(typeof(SwaggerDateConverter))]
        public DateTime? ResponseDate { get; set; }

        /// <summary>
        /// The amount of time (in seconds) it took for the respondent to complete the survey.
        /// </summary>
        /// <value>The amount of time (in seconds) it took for the respondent to complete the survey.</value>
        [DataMember(Name="responseTime", EmitDefaultValue=false)]
        public int? ResponseTime { get; set; }

        /// <summary>
        /// An unordered collection of surveyResponseSurveyLevels. Provides information about the respondents of a survey and how they can be grouped together.
        /// </summary>
        /// <value>An unordered collection of surveyResponseSurveyLevels. Provides information about the respondents of a survey and how they can be grouped together.</value>
        [DataMember(Name="surveyLevels", EmitDefaultValue=false)]
        public List<EdFiSurveyResponseSurveyLevel> SurveyLevels { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiSurveyResponse {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  SurveyResponseIdentifier: ").Append(SurveyResponseIdentifier).Append("\n");
            sb.Append("  ParentReference: ").Append(ParentReference).Append("\n");
            sb.Append("  StaffReference: ").Append(StaffReference).Append("\n");
            sb.Append("  StudentReference: ").Append(StudentReference).Append("\n");
            sb.Append("  SurveyReference: ").Append(SurveyReference).Append("\n");
            sb.Append("  ElectronicMailAddress: ").Append(ElectronicMailAddress).Append("\n");
            sb.Append("  FullName: ").Append(FullName).Append("\n");
            sb.Append("  Location: ").Append(Location).Append("\n");
            sb.Append("  ResponseDate: ").Append(ResponseDate).Append("\n");
            sb.Append("  ResponseTime: ").Append(ResponseTime).Append("\n");
            sb.Append("  SurveyLevels: ").Append(SurveyLevels).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiSurveyResponse);
        }

        /// <summary>
        /// Returns true if EdFiSurveyResponse instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiSurveyResponse to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiSurveyResponse input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.SurveyResponseIdentifier == input.SurveyResponseIdentifier ||
                    (this.SurveyResponseIdentifier != null &&
                    this.SurveyResponseIdentifier.Equals(input.SurveyResponseIdentifier))
                ) && 
                (
                    this.ParentReference == input.ParentReference ||
                    (this.ParentReference != null &&
                    this.ParentReference.Equals(input.ParentReference))
                ) && 
                (
                    this.StaffReference == input.StaffReference ||
                    (this.StaffReference != null &&
                    this.StaffReference.Equals(input.StaffReference))
                ) && 
                (
                    this.StudentReference == input.StudentReference ||
                    (this.StudentReference != null &&
                    this.StudentReference.Equals(input.StudentReference))
                ) && 
                (
                    this.SurveyReference == input.SurveyReference ||
                    (this.SurveyReference != null &&
                    this.SurveyReference.Equals(input.SurveyReference))
                ) && 
                (
                    this.ElectronicMailAddress == input.ElectronicMailAddress ||
                    (this.ElectronicMailAddress != null &&
                    this.ElectronicMailAddress.Equals(input.ElectronicMailAddress))
                ) && 
                (
                    this.FullName == input.FullName ||
                    (this.FullName != null &&
                    this.FullName.Equals(input.FullName))
                ) && 
                (
                    this.Location == input.Location ||
                    (this.Location != null &&
                    this.Location.Equals(input.Location))
                ) && 
                (
                    this.ResponseDate == input.ResponseDate ||
                    (this.ResponseDate != null &&
                    this.ResponseDate.Equals(input.ResponseDate))
                ) && 
                (
                    this.ResponseTime == input.ResponseTime ||
                    (this.ResponseTime != null &&
                    this.ResponseTime.Equals(input.ResponseTime))
                ) && 
                (
                    this.SurveyLevels == input.SurveyLevels ||
                    this.SurveyLevels != null &&
                    this.SurveyLevels.SequenceEqual(input.SurveyLevels)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.SurveyResponseIdentifier != null)
                    hashCode = hashCode * 59 + this.SurveyResponseIdentifier.GetHashCode();
                if (this.ParentReference != null)
                    hashCode = hashCode * 59 + this.ParentReference.GetHashCode();
                if (this.StaffReference != null)
                    hashCode = hashCode * 59 + this.StaffReference.GetHashCode();
                if (this.StudentReference != null)
                    hashCode = hashCode * 59 + this.StudentReference.GetHashCode();
                if (this.SurveyReference != null)
                    hashCode = hashCode * 59 + this.SurveyReference.GetHashCode();
                if (this.ElectronicMailAddress != null)
                    hashCode = hashCode * 59 + this.ElectronicMailAddress.GetHashCode();
                if (this.FullName != null)
                    hashCode = hashCode * 59 + this.FullName.GetHashCode();
                if (this.Location != null)
                    hashCode = hashCode * 59 + this.Location.GetHashCode();
                if (this.ResponseDate != null)
                    hashCode = hashCode * 59 + this.ResponseDate.GetHashCode();
                if (this.ResponseTime != null)
                    hashCode = hashCode * 59 + this.ResponseTime.GetHashCode();
                if (this.SurveyLevels != null)
                    hashCode = hashCode * 59 + this.SurveyLevels.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // SurveyResponseIdentifier (string) maxLength
            if(this.SurveyResponseIdentifier != null && this.SurveyResponseIdentifier.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SurveyResponseIdentifier, length must be less than 60.", new [] { "SurveyResponseIdentifier" });
            }

            // ElectronicMailAddress (string) maxLength
            if(this.ElectronicMailAddress != null && this.ElectronicMailAddress.Length > 128)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ElectronicMailAddress, length must be less than 128.", new [] { "ElectronicMailAddress" });
            }

            // FullName (string) maxLength
            if(this.FullName != null && this.FullName.Length > 80)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for FullName, length must be less than 80.", new [] { "FullName" });
            }

            // Location (string) maxLength
            if(this.Location != null && this.Location.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Location, length must be less than 75.", new [] { "Location" });
            }

            yield break;
        }
    }

}
