/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiXFinanceChartOfAccount
    /// </summary>
    [DataContract]
    public partial class EdFiXFinanceChartOfAccount :  IEquatable<EdFiXFinanceChartOfAccount>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceChartOfAccount" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiXFinanceChartOfAccount() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceChartOfAccount" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="AccountIdentifier">SEA populated Code value for the valid combination of account dimensions under which financials are reported. (required).</param>
        /// <param name="BalanceSheetDimensionReference">BalanceSheetDimensionReference.</param>
        /// <param name="EducationOrganizationReference">EducationOrganizationReference (required).</param>
        /// <param name="FunctionDimensionReference">FunctionDimensionReference.</param>
        /// <param name="FundDimensionReference">FundDimensionReference.</param>
        /// <param name="ObjectDimensionReference">ObjectDimensionReference.</param>
        /// <param name="OperationalUnitDimensionReference">OperationalUnitDimensionReference.</param>
        /// <param name="ProgramDimensionReference">ProgramDimensionReference.</param>
        /// <param name="ProjectDimensionReference">ProjectDimensionReference.</param>
        /// <param name="SourceDimensionReference">SourceDimensionReference.</param>
        /// <param name="AccountName">A descriptive name for the account.</param>
        /// <param name="AccountTypeDescriptor">Type of Account such as Revenue, Expenditure, or Balance Sheet. (required).</param>
        /// <param name="ReportingTags">An unordered collection of chartOfAccountReportingTags. Optional tag for accountability reporting (e.g. ESSA)..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiXFinanceChartOfAccount(string Id = default(string), string AccountIdentifier = default(string), EdFiXFinanceBalanceSheetDimensionReference BalanceSheetDimensionReference = default(EdFiXFinanceBalanceSheetDimensionReference), EdFiEducationOrganizationReference EducationOrganizationReference = default(EdFiEducationOrganizationReference), EdFiXFinanceFunctionDimensionReference FunctionDimensionReference = default(EdFiXFinanceFunctionDimensionReference), EdFiXFinanceFundDimensionReference FundDimensionReference = default(EdFiXFinanceFundDimensionReference), EdFiXFinanceObjectDimensionReference ObjectDimensionReference = default(EdFiXFinanceObjectDimensionReference), EdFiXFinanceOperationalUnitDimensionReference OperationalUnitDimensionReference = default(EdFiXFinanceOperationalUnitDimensionReference), EdFiXFinanceProgramDimensionReference ProgramDimensionReference = default(EdFiXFinanceProgramDimensionReference), EdFiXFinanceProjectDimensionReference ProjectDimensionReference = default(EdFiXFinanceProjectDimensionReference), EdFiXFinanceSourceDimensionReference SourceDimensionReference = default(EdFiXFinanceSourceDimensionReference), string AccountName = default(string), string AccountTypeDescriptor = default(string), List<EdFiXFinanceChartOfAccountReportingTag> ReportingTags = default(List<EdFiXFinanceChartOfAccountReportingTag>), string Etag = default(string))
        {
            // to ensure "AccountIdentifier" is required (not null)
            if (AccountIdentifier == null)
            {
                throw new InvalidDataException("AccountIdentifier is a required property for EdFiXFinanceChartOfAccount and cannot be null");
            }
            else
            {
                this.AccountIdentifier = AccountIdentifier;
            }
            // to ensure "EducationOrganizationReference" is required (not null)
            if (EducationOrganizationReference == null)
            {
                throw new InvalidDataException("EducationOrganizationReference is a required property for EdFiXFinanceChartOfAccount and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = EducationOrganizationReference;
            }
            // to ensure "AccountTypeDescriptor" is required (not null)
            if (AccountTypeDescriptor == null)
            {
                throw new InvalidDataException("AccountTypeDescriptor is a required property for EdFiXFinanceChartOfAccount and cannot be null");
            }
            else
            {
                this.AccountTypeDescriptor = AccountTypeDescriptor;
            }
            this.Id = Id;
            this.BalanceSheetDimensionReference = BalanceSheetDimensionReference;
            this.FunctionDimensionReference = FunctionDimensionReference;
            this.FundDimensionReference = FundDimensionReference;
            this.ObjectDimensionReference = ObjectDimensionReference;
            this.OperationalUnitDimensionReference = OperationalUnitDimensionReference;
            this.ProgramDimensionReference = ProgramDimensionReference;
            this.ProjectDimensionReference = ProjectDimensionReference;
            this.SourceDimensionReference = SourceDimensionReference;
            this.AccountName = AccountName;
            this.ReportingTags = ReportingTags;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// SEA populated Code value for the valid combination of account dimensions under which financials are reported.
        /// </summary>
        /// <value>SEA populated Code value for the valid combination of account dimensions under which financials are reported.</value>
        [DataMember(Name="accountIdentifier", EmitDefaultValue=false)]
        public string AccountIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets BalanceSheetDimensionReference
        /// </summary>
        [DataMember(Name="balanceSheetDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceBalanceSheetDimensionReference BalanceSheetDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// Gets or Sets FunctionDimensionReference
        /// </summary>
        [DataMember(Name="functionDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceFunctionDimensionReference FunctionDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets FundDimensionReference
        /// </summary>
        [DataMember(Name="fundDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceFundDimensionReference FundDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets ObjectDimensionReference
        /// </summary>
        [DataMember(Name="objectDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceObjectDimensionReference ObjectDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets OperationalUnitDimensionReference
        /// </summary>
        [DataMember(Name="operationalUnitDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceOperationalUnitDimensionReference OperationalUnitDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets ProgramDimensionReference
        /// </summary>
        [DataMember(Name="programDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceProgramDimensionReference ProgramDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets ProjectDimensionReference
        /// </summary>
        [DataMember(Name="projectDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceProjectDimensionReference ProjectDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets SourceDimensionReference
        /// </summary>
        [DataMember(Name="sourceDimensionReference", EmitDefaultValue=false)]
        public EdFiXFinanceSourceDimensionReference SourceDimensionReference { get; set; }

        /// <summary>
        /// A descriptive name for the account
        /// </summary>
        /// <value>A descriptive name for the account</value>
        [DataMember(Name="accountName", EmitDefaultValue=false)]
        public string AccountName { get; set; }

        /// <summary>
        /// Type of Account such as Revenue, Expenditure, or Balance Sheet.
        /// </summary>
        /// <value>Type of Account such as Revenue, Expenditure, or Balance Sheet.</value>
        [DataMember(Name="accountTypeDescriptor", EmitDefaultValue=false)]
        public string AccountTypeDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of chartOfAccountReportingTags. Optional tag for accountability reporting (e.g. ESSA).
        /// </summary>
        /// <value>An unordered collection of chartOfAccountReportingTags. Optional tag for accountability reporting (e.g. ESSA).</value>
        [DataMember(Name="reportingTags", EmitDefaultValue=false)]
        public List<EdFiXFinanceChartOfAccountReportingTag> ReportingTags { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiXFinanceChartOfAccount {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AccountIdentifier: ").Append(AccountIdentifier).Append("\n");
            sb.Append("  BalanceSheetDimensionReference: ").Append(BalanceSheetDimensionReference).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  FunctionDimensionReference: ").Append(FunctionDimensionReference).Append("\n");
            sb.Append("  FundDimensionReference: ").Append(FundDimensionReference).Append("\n");
            sb.Append("  ObjectDimensionReference: ").Append(ObjectDimensionReference).Append("\n");
            sb.Append("  OperationalUnitDimensionReference: ").Append(OperationalUnitDimensionReference).Append("\n");
            sb.Append("  ProgramDimensionReference: ").Append(ProgramDimensionReference).Append("\n");
            sb.Append("  ProjectDimensionReference: ").Append(ProjectDimensionReference).Append("\n");
            sb.Append("  SourceDimensionReference: ").Append(SourceDimensionReference).Append("\n");
            sb.Append("  AccountName: ").Append(AccountName).Append("\n");
            sb.Append("  AccountTypeDescriptor: ").Append(AccountTypeDescriptor).Append("\n");
            sb.Append("  ReportingTags: ").Append(ReportingTags).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiXFinanceChartOfAccount);
        }

        /// <summary>
        /// Returns true if EdFiXFinanceChartOfAccount instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiXFinanceChartOfAccount to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiXFinanceChartOfAccount input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AccountIdentifier == input.AccountIdentifier ||
                    (this.AccountIdentifier != null &&
                    this.AccountIdentifier.Equals(input.AccountIdentifier))
                ) && 
                (
                    this.BalanceSheetDimensionReference == input.BalanceSheetDimensionReference ||
                    (this.BalanceSheetDimensionReference != null &&
                    this.BalanceSheetDimensionReference.Equals(input.BalanceSheetDimensionReference))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.FunctionDimensionReference == input.FunctionDimensionReference ||
                    (this.FunctionDimensionReference != null &&
                    this.FunctionDimensionReference.Equals(input.FunctionDimensionReference))
                ) && 
                (
                    this.FundDimensionReference == input.FundDimensionReference ||
                    (this.FundDimensionReference != null &&
                    this.FundDimensionReference.Equals(input.FundDimensionReference))
                ) && 
                (
                    this.ObjectDimensionReference == input.ObjectDimensionReference ||
                    (this.ObjectDimensionReference != null &&
                    this.ObjectDimensionReference.Equals(input.ObjectDimensionReference))
                ) && 
                (
                    this.OperationalUnitDimensionReference == input.OperationalUnitDimensionReference ||
                    (this.OperationalUnitDimensionReference != null &&
                    this.OperationalUnitDimensionReference.Equals(input.OperationalUnitDimensionReference))
                ) && 
                (
                    this.ProgramDimensionReference == input.ProgramDimensionReference ||
                    (this.ProgramDimensionReference != null &&
                    this.ProgramDimensionReference.Equals(input.ProgramDimensionReference))
                ) && 
                (
                    this.ProjectDimensionReference == input.ProjectDimensionReference ||
                    (this.ProjectDimensionReference != null &&
                    this.ProjectDimensionReference.Equals(input.ProjectDimensionReference))
                ) && 
                (
                    this.SourceDimensionReference == input.SourceDimensionReference ||
                    (this.SourceDimensionReference != null &&
                    this.SourceDimensionReference.Equals(input.SourceDimensionReference))
                ) && 
                (
                    this.AccountName == input.AccountName ||
                    (this.AccountName != null &&
                    this.AccountName.Equals(input.AccountName))
                ) && 
                (
                    this.AccountTypeDescriptor == input.AccountTypeDescriptor ||
                    (this.AccountTypeDescriptor != null &&
                    this.AccountTypeDescriptor.Equals(input.AccountTypeDescriptor))
                ) && 
                (
                    this.ReportingTags == input.ReportingTags ||
                    this.ReportingTags != null &&
                    this.ReportingTags.SequenceEqual(input.ReportingTags)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.AccountIdentifier != null)
                    hashCode = hashCode * 59 + this.AccountIdentifier.GetHashCode();
                if (this.BalanceSheetDimensionReference != null)
                    hashCode = hashCode * 59 + this.BalanceSheetDimensionReference.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.FunctionDimensionReference != null)
                    hashCode = hashCode * 59 + this.FunctionDimensionReference.GetHashCode();
                if (this.FundDimensionReference != null)
                    hashCode = hashCode * 59 + this.FundDimensionReference.GetHashCode();
                if (this.ObjectDimensionReference != null)
                    hashCode = hashCode * 59 + this.ObjectDimensionReference.GetHashCode();
                if (this.OperationalUnitDimensionReference != null)
                    hashCode = hashCode * 59 + this.OperationalUnitDimensionReference.GetHashCode();
                if (this.ProgramDimensionReference != null)
                    hashCode = hashCode * 59 + this.ProgramDimensionReference.GetHashCode();
                if (this.ProjectDimensionReference != null)
                    hashCode = hashCode * 59 + this.ProjectDimensionReference.GetHashCode();
                if (this.SourceDimensionReference != null)
                    hashCode = hashCode * 59 + this.SourceDimensionReference.GetHashCode();
                if (this.AccountName != null)
                    hashCode = hashCode * 59 + this.AccountName.GetHashCode();
                if (this.AccountTypeDescriptor != null)
                    hashCode = hashCode * 59 + this.AccountTypeDescriptor.GetHashCode();
                if (this.ReportingTags != null)
                    hashCode = hashCode * 59 + this.ReportingTags.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // AccountIdentifier (string) maxLength
            if(this.AccountIdentifier != null && this.AccountIdentifier.Length > 50)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountIdentifier, length must be less than 50.", new [] { "AccountIdentifier" });
            }

            // AccountName (string) maxLength
            if(this.AccountName != null && this.AccountName.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountName, length must be less than 100.", new [] { "AccountName" });
            }

            // AccountTypeDescriptor (string) maxLength
            if(this.AccountTypeDescriptor != null && this.AccountTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountTypeDescriptor, length must be less than 306.", new [] { "AccountTypeDescriptor" });
            }

            yield break;
        }
    }

}
