/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiXFinanceFundDimensionReference
    /// </summary>
    [DataContract]
    public partial class EdFiXFinanceFundDimensionReference :  IEquatable<EdFiXFinanceFundDimensionReference>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceFundDimensionReference" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiXFinanceFundDimensionReference() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceFundDimensionReference" /> class.
        /// </summary>
        /// <param name="Code">The code representation of the account Fund dimension. (required).</param>
        /// <param name="FiscalYear">The fiscal year for which the account Fund dimension is valid. (required).</param>
        /// <param name="Link">Link.</param>
        public EdFiXFinanceFundDimensionReference(string Code = default(string), int? FiscalYear = default(int?), Link Link = default(Link))
        {
            // to ensure "Code" is required (not null)
            if (Code == null)
            {
                throw new InvalidDataException("Code is a required property for EdFiXFinanceFundDimensionReference and cannot be null");
            }
            else
            {
                this.Code = Code;
            }
            // to ensure "FiscalYear" is required (not null)
            if (FiscalYear == null)
            {
                throw new InvalidDataException("FiscalYear is a required property for EdFiXFinanceFundDimensionReference and cannot be null");
            }
            else
            {
                this.FiscalYear = FiscalYear;
            }
            this.Link = Link;
        }
        
        /// <summary>
        /// The code representation of the account Fund dimension.
        /// </summary>
        /// <value>The code representation of the account Fund dimension.</value>
        [DataMember(Name="code", EmitDefaultValue=false)]
        public string Code { get; set; }

        /// <summary>
        /// The fiscal year for which the account Fund dimension is valid.
        /// </summary>
        /// <value>The fiscal year for which the account Fund dimension is valid.</value>
        [DataMember(Name="fiscalYear", EmitDefaultValue=false)]
        public int? FiscalYear { get; set; }

        /// <summary>
        /// Gets or Sets Link
        /// </summary>
        [DataMember(Name="link", EmitDefaultValue=false)]
        public Link Link { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiXFinanceFundDimensionReference {\n");
            sb.Append("  Code: ").Append(Code).Append("\n");
            sb.Append("  FiscalYear: ").Append(FiscalYear).Append("\n");
            sb.Append("  Link: ").Append(Link).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiXFinanceFundDimensionReference);
        }

        /// <summary>
        /// Returns true if EdFiXFinanceFundDimensionReference instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiXFinanceFundDimensionReference to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiXFinanceFundDimensionReference input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Code == input.Code ||
                    (this.Code != null &&
                    this.Code.Equals(input.Code))
                ) && 
                (
                    this.FiscalYear == input.FiscalYear ||
                    (this.FiscalYear != null &&
                    this.FiscalYear.Equals(input.FiscalYear))
                ) && 
                (
                    this.Link == input.Link ||
                    (this.Link != null &&
                    this.Link.Equals(input.Link))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Code != null)
                    hashCode = hashCode * 59 + this.Code.GetHashCode();
                if (this.FiscalYear != null)
                    hashCode = hashCode * 59 + this.FiscalYear.GetHashCode();
                if (this.Link != null)
                    hashCode = hashCode * 59 + this.Link.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // Code (string) maxLength
            if(this.Code != null && this.Code.Length > 16)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Code, length must be less than 16.", new [] { "Code" });
            }

            yield break;
        }
    }

}
