/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiXFinanceLocalAccount
    /// </summary>
    [DataContract]
    public partial class EdFiXFinanceLocalAccount :  IEquatable<EdFiXFinanceLocalAccount>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceLocalAccount" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiXFinanceLocalAccount() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceLocalAccount" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="AccountIdentifier">Code value for the valid combination of account dimensions by LEA under which financials are reported. (required).</param>
        /// <param name="ChartOfAccountReference">ChartOfAccountReference (required).</param>
        /// <param name="EducationOrganizationReference">EducationOrganizationReference (required).</param>
        /// <param name="AccountName">A descriptive name for the account.</param>
        /// <param name="ReportingTags">An unordered collection of localAccountReportingTags. Optional tag for accountability reporting (e.g. ESSA)..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiXFinanceLocalAccount(string Id = default(string), string AccountIdentifier = default(string), EdFiXFinanceChartOfAccountReference ChartOfAccountReference = default(EdFiXFinanceChartOfAccountReference), EdFiEducationOrganizationReference EducationOrganizationReference = default(EdFiEducationOrganizationReference), string AccountName = default(string), List<EdFiXFinanceLocalAccountReportingTag> ReportingTags = default(List<EdFiXFinanceLocalAccountReportingTag>), string Etag = default(string))
        {
            // to ensure "AccountIdentifier" is required (not null)
            if (AccountIdentifier == null)
            {
                throw new InvalidDataException("AccountIdentifier is a required property for EdFiXFinanceLocalAccount and cannot be null");
            }
            else
            {
                this.AccountIdentifier = AccountIdentifier;
            }
            // to ensure "ChartOfAccountReference" is required (not null)
            if (ChartOfAccountReference == null)
            {
                throw new InvalidDataException("ChartOfAccountReference is a required property for EdFiXFinanceLocalAccount and cannot be null");
            }
            else
            {
                this.ChartOfAccountReference = ChartOfAccountReference;
            }
            // to ensure "EducationOrganizationReference" is required (not null)
            if (EducationOrganizationReference == null)
            {
                throw new InvalidDataException("EducationOrganizationReference is a required property for EdFiXFinanceLocalAccount and cannot be null");
            }
            else
            {
                this.EducationOrganizationReference = EducationOrganizationReference;
            }
            this.Id = Id;
            this.AccountName = AccountName;
            this.ReportingTags = ReportingTags;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// Code value for the valid combination of account dimensions by LEA under which financials are reported.
        /// </summary>
        /// <value>Code value for the valid combination of account dimensions by LEA under which financials are reported.</value>
        [DataMember(Name="accountIdentifier", EmitDefaultValue=false)]
        public string AccountIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets ChartOfAccountReference
        /// </summary>
        [DataMember(Name="chartOfAccountReference", EmitDefaultValue=false)]
        public EdFiXFinanceChartOfAccountReference ChartOfAccountReference { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name="educationOrganizationReference", EmitDefaultValue=false)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// A descriptive name for the account
        /// </summary>
        /// <value>A descriptive name for the account</value>
        [DataMember(Name="accountName", EmitDefaultValue=false)]
        public string AccountName { get; set; }

        /// <summary>
        /// An unordered collection of localAccountReportingTags. Optional tag for accountability reporting (e.g. ESSA).
        /// </summary>
        /// <value>An unordered collection of localAccountReportingTags. Optional tag for accountability reporting (e.g. ESSA).</value>
        [DataMember(Name="reportingTags", EmitDefaultValue=false)]
        public List<EdFiXFinanceLocalAccountReportingTag> ReportingTags { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiXFinanceLocalAccount {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AccountIdentifier: ").Append(AccountIdentifier).Append("\n");
            sb.Append("  ChartOfAccountReference: ").Append(ChartOfAccountReference).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  AccountName: ").Append(AccountName).Append("\n");
            sb.Append("  ReportingTags: ").Append(ReportingTags).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiXFinanceLocalAccount);
        }

        /// <summary>
        /// Returns true if EdFiXFinanceLocalAccount instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiXFinanceLocalAccount to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiXFinanceLocalAccount input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AccountIdentifier == input.AccountIdentifier ||
                    (this.AccountIdentifier != null &&
                    this.AccountIdentifier.Equals(input.AccountIdentifier))
                ) && 
                (
                    this.ChartOfAccountReference == input.ChartOfAccountReference ||
                    (this.ChartOfAccountReference != null &&
                    this.ChartOfAccountReference.Equals(input.ChartOfAccountReference))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.AccountName == input.AccountName ||
                    (this.AccountName != null &&
                    this.AccountName.Equals(input.AccountName))
                ) && 
                (
                    this.ReportingTags == input.ReportingTags ||
                    this.ReportingTags != null &&
                    this.ReportingTags.SequenceEqual(input.ReportingTags)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.AccountIdentifier != null)
                    hashCode = hashCode * 59 + this.AccountIdentifier.GetHashCode();
                if (this.ChartOfAccountReference != null)
                    hashCode = hashCode * 59 + this.ChartOfAccountReference.GetHashCode();
                if (this.EducationOrganizationReference != null)
                    hashCode = hashCode * 59 + this.EducationOrganizationReference.GetHashCode();
                if (this.AccountName != null)
                    hashCode = hashCode * 59 + this.AccountName.GetHashCode();
                if (this.ReportingTags != null)
                    hashCode = hashCode * 59 + this.ReportingTags.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // AccountIdentifier (string) maxLength
            if(this.AccountIdentifier != null && this.AccountIdentifier.Length > 50)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountIdentifier, length must be less than 50.", new [] { "AccountIdentifier" });
            }

            // AccountName (string) maxLength
            if(this.AccountName != null && this.AccountName.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountName, length must be less than 100.", new [] { "AccountName" });
            }

            yield break;
        }
    }

}
