/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiXFinanceSourceDimension
    /// </summary>
    [DataContract]
    public partial class EdFiXFinanceSourceDimension :  IEquatable<EdFiXFinanceSourceDimension>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceSourceDimension" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiXFinanceSourceDimension() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiXFinanceSourceDimension" /> class.
        /// </summary>
        /// <param name="Id">Id.</param>
        /// <param name="Code">The code representation of the account Source dimension. (required).</param>
        /// <param name="FiscalYear">The fiscal year for which the account Source dimension is valid. (required).</param>
        /// <param name="CodeName">A description of the account Source dimension..</param>
        /// <param name="ReportingTags">An unordered collection of sourceDimensionReportingTags. Optional tag for accountability reporting (e.g. ESSA)..</param>
        /// <param name="Etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiXFinanceSourceDimension(string Id = default(string), string Code = default(string), int? FiscalYear = default(int?), string CodeName = default(string), List<EdFiXFinanceSourceDimensionReportingTag> ReportingTags = default(List<EdFiXFinanceSourceDimensionReportingTag>), string Etag = default(string))
        {
            // to ensure "Code" is required (not null)
            if (Code == null)
            {
                throw new InvalidDataException("Code is a required property for EdFiXFinanceSourceDimension and cannot be null");
            }
            else
            {
                this.Code = Code;
            }
            // to ensure "FiscalYear" is required (not null)
            if (FiscalYear == null)
            {
                throw new InvalidDataException("FiscalYear is a required property for EdFiXFinanceSourceDimension and cannot be null");
            }
            else
            {
                this.FiscalYear = FiscalYear;
            }
            this.Id = Id;
            this.CodeName = CodeName;
            this.ReportingTags = ReportingTags;
            this.Etag = Etag;
        }
        
        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name="id", EmitDefaultValue=false)]
        public string Id { get; set; }

        /// <summary>
        /// The code representation of the account Source dimension.
        /// </summary>
        /// <value>The code representation of the account Source dimension.</value>
        [DataMember(Name="code", EmitDefaultValue=false)]
        public string Code { get; set; }

        /// <summary>
        /// The fiscal year for which the account Source dimension is valid.
        /// </summary>
        /// <value>The fiscal year for which the account Source dimension is valid.</value>
        [DataMember(Name="fiscalYear", EmitDefaultValue=false)]
        public int? FiscalYear { get; set; }

        /// <summary>
        /// A description of the account Source dimension.
        /// </summary>
        /// <value>A description of the account Source dimension.</value>
        [DataMember(Name="codeName", EmitDefaultValue=false)]
        public string CodeName { get; set; }

        /// <summary>
        /// An unordered collection of sourceDimensionReportingTags. Optional tag for accountability reporting (e.g. ESSA).
        /// </summary>
        /// <value>An unordered collection of sourceDimensionReportingTags. Optional tag for accountability reporting (e.g. ESSA).</value>
        [DataMember(Name="reportingTags", EmitDefaultValue=false)]
        public List<EdFiXFinanceSourceDimensionReportingTag> ReportingTags { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name="_etag", EmitDefaultValue=false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiXFinanceSourceDimension {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  Code: ").Append(Code).Append("\n");
            sb.Append("  FiscalYear: ").Append(FiscalYear).Append("\n");
            sb.Append("  CodeName: ").Append(CodeName).Append("\n");
            sb.Append("  ReportingTags: ").Append(ReportingTags).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiXFinanceSourceDimension);
        }

        /// <summary>
        /// Returns true if EdFiXFinanceSourceDimension instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiXFinanceSourceDimension to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiXFinanceSourceDimension input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.Code == input.Code ||
                    (this.Code != null &&
                    this.Code.Equals(input.Code))
                ) && 
                (
                    this.FiscalYear == input.FiscalYear ||
                    (this.FiscalYear != null &&
                    this.FiscalYear.Equals(input.FiscalYear))
                ) && 
                (
                    this.CodeName == input.CodeName ||
                    (this.CodeName != null &&
                    this.CodeName.Equals(input.CodeName))
                ) && 
                (
                    this.ReportingTags == input.ReportingTags ||
                    this.ReportingTags != null &&
                    this.ReportingTags.SequenceEqual(input.ReportingTags)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                    hashCode = hashCode * 59 + this.Id.GetHashCode();
                if (this.Code != null)
                    hashCode = hashCode * 59 + this.Code.GetHashCode();
                if (this.FiscalYear != null)
                    hashCode = hashCode * 59 + this.FiscalYear.GetHashCode();
                if (this.CodeName != null)
                    hashCode = hashCode * 59 + this.CodeName.GetHashCode();
                if (this.ReportingTags != null)
                    hashCode = hashCode * 59 + this.ReportingTags.GetHashCode();
                if (this.Etag != null)
                    hashCode = hashCode * 59 + this.Etag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // Code (string) maxLength
            if(this.Code != null && this.Code.Length > 16)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Code, length must be less than 16.", new [] { "Code" });
            }

            // CodeName (string) maxLength
            if(this.CodeName != null && this.CodeName.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CodeName, length must be less than 100.", new [] { "CodeName" });
            }

            yield break;
        }
    }

}
