/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// WiGraduationPlanExtension
    /// </summary>
    [DataContract]
    public partial class WiGraduationPlanExtension :  IEquatable<WiGraduationPlanExtension>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WiGraduationPlanExtension" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected WiGraduationPlanExtension() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="WiGraduationPlanExtension" /> class.
        /// </summary>
        /// <param name="CivicsExam">A civics exam is required- Yes -1 or No - 0 (required).</param>
        /// <param name="CommunityService">Total hours of community service required for graduation (required).</param>
        /// <param name="MiscellaneousRequiredCredits">Clarification of requirements for Miscellaneous Required Credits.</param>
        /// <param name="NonHsHealthEducationCredits">Credits of Health Education Fulfilled in Grade 7 and/or Grade 8 (required).</param>
        public WiGraduationPlanExtension(bool? CivicsExam = default(bool?), double? CommunityService = default(double?), string MiscellaneousRequiredCredits = default(string), double? NonHsHealthEducationCredits = default(double?))
        {
            // to ensure "CivicsExam" is required (not null)
            if (CivicsExam == null)
            {
                throw new InvalidDataException("CivicsExam is a required property for WiGraduationPlanExtension and cannot be null");
            }
            else
            {
                this.CivicsExam = CivicsExam;
            }
            // to ensure "CommunityService" is required (not null)
            if (CommunityService == null)
            {
                throw new InvalidDataException("CommunityService is a required property for WiGraduationPlanExtension and cannot be null");
            }
            else
            {
                this.CommunityService = CommunityService;
            }
            // to ensure "NonHsHealthEducationCredits" is required (not null)
            if (NonHsHealthEducationCredits == null)
            {
                throw new InvalidDataException("NonHsHealthEducationCredits is a required property for WiGraduationPlanExtension and cannot be null");
            }
            else
            {
                this.NonHsHealthEducationCredits = NonHsHealthEducationCredits;
            }
            this.MiscellaneousRequiredCredits = MiscellaneousRequiredCredits;
        }
        
        /// <summary>
        /// A civics exam is required- Yes -1 or No - 0
        /// </summary>
        /// <value>A civics exam is required- Yes -1 or No - 0</value>
        [DataMember(Name="civicsExam", EmitDefaultValue=false)]
        public bool? CivicsExam { get; set; }

        /// <summary>
        /// Total hours of community service required for graduation
        /// </summary>
        /// <value>Total hours of community service required for graduation</value>
        [DataMember(Name="communityService", EmitDefaultValue=false)]
        public double? CommunityService { get; set; }

        /// <summary>
        /// Clarification of requirements for Miscellaneous Required Credits
        /// </summary>
        /// <value>Clarification of requirements for Miscellaneous Required Credits</value>
        [DataMember(Name="miscellaneousRequiredCredits", EmitDefaultValue=false)]
        public string MiscellaneousRequiredCredits { get; set; }

        /// <summary>
        /// Credits of Health Education Fulfilled in Grade 7 and/or Grade 8
        /// </summary>
        /// <value>Credits of Health Education Fulfilled in Grade 7 and/or Grade 8</value>
        [DataMember(Name="nonHsHealthEducationCredits", EmitDefaultValue=false)]
        public double? NonHsHealthEducationCredits { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class WiGraduationPlanExtension {\n");
            sb.Append("  CivicsExam: ").Append(CivicsExam).Append("\n");
            sb.Append("  CommunityService: ").Append(CommunityService).Append("\n");
            sb.Append("  MiscellaneousRequiredCredits: ").Append(MiscellaneousRequiredCredits).Append("\n");
            sb.Append("  NonHsHealthEducationCredits: ").Append(NonHsHealthEducationCredits).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as WiGraduationPlanExtension);
        }

        /// <summary>
        /// Returns true if WiGraduationPlanExtension instances are equal
        /// </summary>
        /// <param name="input">Instance of WiGraduationPlanExtension to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(WiGraduationPlanExtension input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.CivicsExam == input.CivicsExam ||
                    (this.CivicsExam != null &&
                    this.CivicsExam.Equals(input.CivicsExam))
                ) && 
                (
                    this.CommunityService == input.CommunityService ||
                    (this.CommunityService != null &&
                    this.CommunityService.Equals(input.CommunityService))
                ) && 
                (
                    this.MiscellaneousRequiredCredits == input.MiscellaneousRequiredCredits ||
                    (this.MiscellaneousRequiredCredits != null &&
                    this.MiscellaneousRequiredCredits.Equals(input.MiscellaneousRequiredCredits))
                ) && 
                (
                    this.NonHsHealthEducationCredits == input.NonHsHealthEducationCredits ||
                    (this.NonHsHealthEducationCredits != null &&
                    this.NonHsHealthEducationCredits.Equals(input.NonHsHealthEducationCredits))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.CivicsExam != null)
                    hashCode = hashCode * 59 + this.CivicsExam.GetHashCode();
                if (this.CommunityService != null)
                    hashCode = hashCode * 59 + this.CommunityService.GetHashCode();
                if (this.MiscellaneousRequiredCredits != null)
                    hashCode = hashCode * 59 + this.MiscellaneousRequiredCredits.GetHashCode();
                if (this.NonHsHealthEducationCredits != null)
                    hashCode = hashCode * 59 + this.NonHsHealthEducationCredits.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // MiscellaneousRequiredCredits (string) maxLength
            if(this.MiscellaneousRequiredCredits != null && this.MiscellaneousRequiredCredits.Length > 900)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for MiscellaneousRequiredCredits, length must be less than 900.", new [] { "MiscellaneousRequiredCredits" });
            }

            yield break;
        }
    }

}
